@@ -351,7 +351,7 @@ FILE *append_newstage(const char *mailboxname, time_t internaldate,
  */
 int append_fromstage(struct appendstate *as, struct body **body,
 		     struct stagemsg *stage, time_t internaldate,
-		     const char **flag, int nflags, int nolink)
+		     const strarray_t *flags, int nolink)
 {
     struct mailbox *mailbox = as->mailbox;
     struct index_record message_index;
@@ -465,33 +465,34 @@ int append_fromstage(struct appendstate *as, struct body **body,
     }
 
     /* Handle flags the user wants to set in the message */
-    for (i = 0; i < nflags; i++) {
-	if (!strcmp(flag[i], "\\seen")) {
+    for (i = 0; flags && i < flags->count ; i++) {
+	const char *flag = strarray_nth(flags, i);
+	if (!strcmp(flag, "\\seen")) {
 	    append_setseen(as, &message_index);
 	}
-	else if (!strcmp(flag[i], "\\deleted")) {
+	else if (!strcmp(flag, "\\deleted")) {
 	    if (as->myrights & ACL_DELETEMSG) {
 		message_index.system_flags |= FLAG_DELETED;
 	    }
 	}
-	else if (!strcmp(flag[i], "\\draft")) {
+	else if (!strcmp(flag, "\\draft")) {
 	    if (as->myrights & ACL_WRITE) {
 		message_index.system_flags |= FLAG_DRAFT;
 	    }
 	}
-	else if (!strcmp(flag[i], "\\flagged")) {
+	else if (!strcmp(flag, "\\flagged")) {
 	    if (as->myrights & ACL_WRITE) {
 		message_index.system_flags |= FLAG_FLAGGED;
 	    }
 	}
-	else if (!strcmp(flag[i], "\\answered")) {
+	else if (!strcmp(flag, "\\answered")) {
 	    if (as->myrights & ACL_WRITE) {
 		message_index.system_flags |= FLAG_ANSWERED;
 	    }
 	}
 	else if (as->myrights & ACL_WRITE) {
 	    /* User flag */
-	    r = mailbox_user_flag(mailbox, flag[i], &userflag);
+	    r = mailbox_user_flag(mailbox, flag, &userflag);
 	    if (!r) 
 		message_index.user_flags[userflag/32] |= 1<<(userflag&31);
 	}
@@ -734,7 +735,7 @@ static int append_addseen(struct mailbox *mailbox,
 {
     int r;
     struct seen *seendb = NULL;
-    struct seendata sd;
+    struct seendata sd = SEENDATA_INITIALIZER;
     struct seqset *oldseen;
 
     if (!newseen->len)
@@ -748,7 +749,7 @@ static int append_addseen(struct mailbox *mailbox,
 
     /* parse the old sequence */
     oldseen = seqset_parse(sd.seenuids, NULL, mailbox->i.last_uid);
-    free(sd.seenuids);
+    seen_freedata(&sd);
 
     /* add the extra items */
     seqset_join(oldseen, newseen);
@@ -758,7 +759,7 @@ static int append_addseen(struct mailbox *mailbox,
     /* and write it out */
     sd.lastchange = time(NULL);
     r = seen_write(seendb, mailbox->uniqueid, &sd);
-    free(sd.seenuids);
+    seen_freedata(&sd);
 
  done:
     seen_close(&seendb);
