@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: append.h,v 1.33 2009/03/16 00:17:14 brong Exp $
+ * $Id: append.h,v 1.34 2010/01/06 17:01:30 murch Exp $
  */
 
 #ifndef INCLUDED_APPEND_H
@@ -48,17 +48,20 @@
 #include "mailbox.h"
 #include "message.h"
 #include "prot.h"
+#include "sequence.h"
+#include "strarray.h"
 
 struct copymsg {
     unsigned long uid;
     time_t internaldate;
     time_t sentdate;
+    time_t gmtime;
     unsigned long size;
     unsigned long header_size;
     unsigned long content_lines;
     unsigned long cache_version;
-    const char *cache_begin;
-    int cache_len;		/* 0 if need to copy & parse message */
+    unsigned long cache_crc;
+    struct cacherecord crec;
     int seen;
     struct message_guid guid;
     bit32 system_flags;
@@ -69,56 +72,42 @@ struct copymsg {
    clients to stack-allocate it */
 struct appendstate {
     /* mailbox we're appending to */
-    struct mailbox m;
+    struct mailbox *mailbox;
+    int myrights;
     char userid[MAX_MAILBOX_BUFFER];
 
     enum { APPEND_READY, APPEND_DONE } s;
 				/* current state of append */
 
-    /* if we abort, where should we truncate the cache file? */
-    unsigned long orig_cache_size;
-
-    int writeheader;		/* did we change the mailbox header? */
-
     int nummsg;    /* number of messages appended pending commit.
-		      from m.last_uid + 1 ... m.last_uid + nummsg */
-
-    /* summary information to change on commit */
-    int numanswered, numdeleted, numflagged;
+		      from as->baseuid ... m.baseuid + nummsg - 1 */
+    unsigned baseuid; 
 
     /* set seen on these message on commit */
-    char *seen_msgrange;
-    int seen_alloced;
-
-    /* the amount of quota we've used so far in this append */
-    uquota_t quota_used;
-
-    /* txn for updating quota */
-    struct txn *tid;
+    int internalseen;
+    struct seqset *seen_seq;
 };
 
 /* add helper function to determine uid range appended? */
 
 struct stagemsg;
 
-extern int append_check(const char *name, int format, 
+extern int append_check(const char *name,
 			struct auth_state *auth_state,
 			long aclcheck, quota_t quotacheck);
 
 /* appendstate must be allocated by client */
-extern int append_setup(struct appendstate *mailbox, const char *name,
-			int format, 
+extern int append_setup(struct appendstate *as, const char *name,
 			const char *userid, struct auth_state *auth_state,
 			long aclcheck, quota_t quotacheck);
 
-extern int append_commit(struct appendstate *mailbox,
+extern int append_commit(struct appendstate *as,
 			 quota_t quotacheck,
 			 unsigned long *uidvalidity, 
 			 unsigned long *startuid, 
-			 unsigned long *num);
-extern int append_abort(struct appendstate *mailbox);
-
-int append_stageparts(struct stagemsg *stagep);
+			 unsigned long *num,
+			 struct mailbox **mailboxptr);
+extern int append_abort(struct appendstate *as);
 
 /* creates a new stage and returns stage file corresponding to mailboxname */
 extern FILE *append_newstage(const char *mailboxname, time_t internaldate,
@@ -127,7 +116,7 @@ extern FILE *append_newstage(const char *mailboxname, time_t internaldate,
 /* adds a new mailbox to the stage initially created by append_newstage() */
 extern int append_fromstage(struct appendstate *mailbox, struct body **body,
 			    struct stagemsg *stage, time_t internaldate,
-			    const char **flag, int nflags, int nolink);
+			    const strarray_t *flags, int nolink);
 
 /* removes the stage (frees memory, deletes the staging files) */
 extern int append_removestage(struct stagemsg *stage);
