@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: arbitron.c,v 1.47 2009/02/09 05:01:56 brong Exp $
+ * $Id: arbitron.c,v 1.48 2010/01/06 17:01:30 murch Exp $
  */
 
 #include <config.h>
@@ -109,7 +109,7 @@ int long_report = 0;
 static struct namespace arb_namespace;
 
 /* forward declarations */
-void usage(void);
+void usage(void) __attribute__((noreturn));
 void run_users(void);
 void make_report(char *key, void *data, void *rock);
 void process_seen(const char *path, const char *user);
@@ -203,7 +203,7 @@ int main(int argc,char **argv)
     
     /* Set namespace -- force standard (internal) */
     if ((r = mboxname_init_namespace(&arb_namespace, 1)) != 0) {
-	syslog(LOG_ERR, error_message(r));
+	syslog(LOG_ERR, "%s", error_message(r));
 	fatal(error_message(r), EC_CONFIG);
     }
 
@@ -265,25 +265,23 @@ int do_mailbox(const char *name, int matchlen __attribute__((unused)),
 	       void *rock __attribute__((unused)))
 {
     int r;
-    struct mailbox mbox;
+    struct mailbox *mailbox = NULL;
 
-    r = mailbox_open_header(name, NULL, &mbox);
-    if(!r) {
-	struct arb_mailbox_data *d = mpool_malloc(arb_pool,
-						  sizeof(struct arb_mailbox_data));
+    r = mailbox_open_irl(name, &mailbox);
+    if (r) return 0;
+
+    struct arb_mailbox_data *d = mpool_malloc(arb_pool,
+					      sizeof(struct arb_mailbox_data));
     
-	d->nreaders = 0;
-	d->nsubscribers = 0;
-	d->readers = NULL;
-	d->subscribers = NULL;
+    d->nreaders = 0;
+    d->nsubscribers = 0;
+    d->readers = NULL;
+    d->subscribers = NULL;
 
-/*	printf("inserting %s (key %s)\n", name, mbox.uniqueid); */
+    hash_insert(mailbox->uniqueid, d, &mailbox_table);
+    hash_insert(name, d, &mboxname_table);
 
-	hash_insert(mbox.uniqueid, d, &mailbox_table);
-	hash_insert(name, d, &mboxname_table);
-
-	mailbox_close(&mbox);
-    }
+    mailbox_close(&mailbox);
 
     return 0;
 }
