@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: ctl_mboxlist.c,v 1.65 2009/03/31 04:11:15 brong Exp $
+ * $Id: ctl_mboxlist.c,v 1.68 2010/06/15 03:10:53 selsky Exp $
  */
 
 /* currently doesn't catch signals; probably SHOULD */
@@ -74,6 +74,7 @@
 #include <sasl/sasl.h>
 
 #include "assert.h"
+#include "annotate.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "global.h"
@@ -137,6 +138,7 @@ static int interactive = 0;
  * a) mupdate server thinks *we* host
  *    -> Because we were called, this is the case, provided we
  *    -> gave the prefix parameter to the remote.
+ *    -> (And assuming bugs don't exist.)
  * b) we do not actually host
  *
  * if that's the case, enqueue a delete
@@ -182,7 +184,7 @@ static int dump_cb(void *rockp,
 		   const char *data, int datalen)
 {
     struct dumprock *d = (struct dumprock *) rockp;
-    int r;
+    int r = 0;
     char *p;
     char *name, *part, *acl;
     int mbtype;
@@ -279,12 +281,12 @@ static int dump_cb(void *rockp,
 	     * mailbox, and if it is living somewhere else, delete the local
 	     * data, if it is NOT living somewhere else, recreate it in
 	     * mupdate */
-	    struct mupdate_mailboxdata *unused_mbdata;
+	    struct mupdate_mailboxdata *mdata;
 
 	    /* if this is okay, we found it (so it is on another host, since
 	     * it wasn't in our list in this position) */
 	    if(!local_authoritative &&
-	       !mupdate_find(d->h, name, &unused_mbdata)) {
+	       !mupdate_find(d->h, name, &mdata)) {
 		/* since it lives on another server, schedule it for a wipe */
 		struct mb_node *next;
 
@@ -294,8 +296,24 @@ static int dump_cb(void *rockp,
 		 * if wrong, we'll end up removing the authoritative
 		 * mailbox.
 		 */
-		assert(strcmp(realpart, unused_mbdata->server));
+		if (strcmp(realpart, mdata->server) == 0 ) {
+		    if ( act_head ) {
+			fprintf( stderr, "mupdate said: %s %s %s\n",
+			    act_head->mailbox, act_head->server, act_head->acl );
+		    }
+		    fprintf( stderr, "mailboxes.db said: %s %s %s\n",
+			    name, realpart, acl );
+		    fprintf( stderr, "mupdate says: %s %s %s\n",
+			    mdata->mailbox, mdata->server, mdata->acl );
+		    fatal("mupdate said not us before it said us", EC_SOFTWARE);
+		}
 		
+		/*
+		 * Where does "unified" murder fit into ctl_mboxlist?
+		 * 1. Only check locally hosted mailboxes.
+		 * 2. Check everything.
+		 * Either way, this check is just wrong!
+		 */
 		if (config_mupdate_config != 
 		    IMAP_ENUM_MUPDATE_CONFIG_UNIFIED) {
 		    /* But not for a unified configuration */
@@ -351,11 +369,12 @@ static int dump_cb(void *rockp,
 	
 	if(r == MUPDATE_NOCONN) {
 	    fprintf(stderr, "permanant failure storing '%s'\n", name);
-	    return IMAP_IOERROR;
+	    r = IMAP_IOERROR;
 	} else if (r == MUPDATE_FAIL) {
 	    fprintf(stderr,
 		    "temporary failure storing '%s' (update continuing)",
 		    name);
+	    r = 0;
 	}
 	    
 	break;
@@ -370,7 +389,7 @@ static int dump_cb(void *rockp,
     free(part);
     free(acl);
 
-    return 0;
+    return r;
 }
 
 /*
@@ -469,14 +488,13 @@ void do_dump(enum mboxop op, const char *part, int purge)
     if(op == M_POPULATE) {
 	/* Remove MBTYPE_MOVING flags (unflag_head) */
 	while(unflag_head) {
+	    struct mboxlist_entry mbentry;
 	    struct mb_node *me = unflag_head;
-	    int type;
-	    char *part, *acl, *newpart;
+	    char *newpart;
 	    
 	    unflag_head = unflag_head->next;
 	    
-	    ret = mboxlist_detail(me->mailbox, &type, NULL, NULL,
-				  &part, &acl, NULL);
+	    ret = mboxlist_lookup(me->mailbox, &mbentry, NULL);
 	    if(ret) {
 		fprintf(stderr,
 			"couldn't perform lookup to un-remote-flag %s\n",
@@ -485,12 +503,13 @@ void do_dump(enum mboxop op, const char *part, int purge)
 	    }
 
 	    /* Reset the partition! */
-	    newpart = strchr(part, '!');
-	    if(!newpart) newpart = part;
+	    newpart = strchr(mbentry.partition, '!');
+	    if(!newpart) newpart = mbentry.partition;
 	    else newpart++;
 
-	    ret = mboxlist_update(me->mailbox, type & ~MBTYPE_MOVING,
-				  newpart, acl, 1);
+	    /* XXX - FIXME, pull uniqueid from detail */
+	    ret = mboxlist_update(me->mailbox, mbentry.mbtype & ~MBTYPE_MOVING,
+				  newpart, mbentry.acl, 1);
 	    if(ret) {
 		fprintf(stderr,
 			"couldn't perform update to un-remote-flag %s\n",
@@ -499,8 +518,8 @@ void do_dump(enum mboxop op, const char *part, int purge)
 	    } 
 	    
 	    /* force a push to mupdate */
-	    snprintf(buf, sizeof(buf), "%s!%s", config_servername, part);
-	    ret = mupdate_activate(d.h, me->mailbox, buf, acl);
+	    snprintf(buf, sizeof(buf), "%s!%s", config_servername, newpart);
+	    ret = mupdate_activate(d.h, me->mailbox, buf, mbentry.acl);
 	    if(ret) {
 		fprintf(stderr,
 			"couldn't perform mupdatepush to un-remote-flag %s\n",
@@ -533,8 +552,7 @@ void do_dump(enum mboxop op, const char *part, int purge)
 	    wipe_head = wipe_head->next;
 	    
 	    ret = mboxlist_deletemailbox(me->mailbox, 1, "", NULL, 0, 1, 1);
-	    if(!ret) sync_log_mailbox(me->mailbox);
-	    if(ret) {
+	    if (ret) {
 		fprintf(stderr, "couldn't delete defunct mailbox %s\n",
 			me->mailbox);
 		exit(1);
@@ -606,7 +624,8 @@ void do_undump(void)
 	}
 
 	key = name; keylen = strlen(key);
-	data = mboxlist_makeentry(mbtype, partition, acl); datalen = strlen(data);
+	data = mboxlist_makeentry(mbtype, partition, acl);
+	datalen = strlen(data);
 	
 	tries = 0;
     retry:
@@ -866,15 +885,9 @@ void do_verify(void)
 
 	if (!(dirp = opendir(found.data[i].path))) continue;
 	while ((dirent = readdir(dirp))) {
-	    struct mailbox mailbox;
-
 	    if (dirent->d_name[0] == '.') continue;
-	    else if (!strcmp(dirent->d_name, FNAME_HEADER+1) &&
-		     !mailbox_open_header_path(found.data[i].mboxname,
-					       found.data[i].path,
-					       found.data[i].path,
-					       "", NULL, &mailbox, 1)) {
-		mailbox_close(&mailbox);
+	    else if (!strcmp(dirent->d_name, FNAME_HEADER+1)) {
+		/* XXX - check that it can be opened */
 		found.data[i].type |= MBOX;
 	    }
 	    else if (!strchr(dirent->d_name, '.') ||
@@ -920,13 +933,13 @@ void do_verify(void)
 void usage(void)
 {
     fprintf(stderr, "DUMP:\n");
-    fprintf(stderr, "  ctl_mboxlist [-C <alt_config>] -d [-x] [-f filename] [-p partition]\n");
+    fprintf(stderr, "  ctl_mboxlist [-C <alt_config>] -d [-x] [-p partition] [-f filename]\n");
     fprintf(stderr, "UNDUMP:\n");
     fprintf(stderr,
 	    "  ctl_mboxlist [-C <alt_config>] -u [-f filename]"
 	    "    [< mboxlist.dump]\n");
     fprintf(stderr, "MUPDATE populate:\n");
-    fprintf(stderr, "  ctl_mboxlist [-C <alt_config>] -m [-a] [-w] [-f filename]\n");
+    fprintf(stderr, "  ctl_mboxlist [-C <alt_config>] -m [-a] [-w] [-i] [-f filename]\n");
     fprintf(stderr, "VERIFY:\n");
     fprintf(stderr, "  ctl_mboxlist [-C <alt_config>] -v [-f filename]\n");
     exit(1);
@@ -1054,11 +1067,17 @@ int main(int argc, char *argv[])
 	mboxlist_init(0);
 	mboxlist_open(mboxdb_fname);
 
- 	quotadb_init(0);
- 	quotadb_open(NULL);
-	
+	quotadb_init(0);
+	quotadb_open(NULL);
+
+	annotatemore_init(0, NULL, NULL);
+	annotatemore_open(NULL);
+
 	do_dump(op, partition, dopurge);
 
+	annotatemore_close();
+	annotatemore_done();
+
 	quotadb_close();
 	quotadb_done();
 	
@@ -1070,11 +1089,17 @@ int main(int argc, char *argv[])
 	mboxlist_init(0);
 	mboxlist_open(mboxdb_fname);
 
- 	quotadb_init(0);
- 	quotadb_open(NULL);
+	quotadb_init(0);
+	quotadb_open(NULL);
+
+	annotatemore_init(0, NULL, NULL);
+	annotatemore_open(NULL);
 
 	do_undump();
 
+	annotatemore_close();
+	annotatemore_done();
+
 	quotadb_close();
 	quotadb_done();
 	
