@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: fetchnews.c,v 1.21 2008/08/27 08:28:46 selsky Exp $
+ * $Id: fetchnews.c,v 1.22 2010/01/06 17:01:31 murch Exp $
  */
 
 #include <config.h>
@@ -62,7 +62,7 @@
 #include "exitcodes.h"
 #include "global.h"
 #include "gmtoff.h"
-#include "lock.h"
+#include "cyr_lock.h"
 #include "prot.h"
 #include "util.h"
 #include "xmalloc.h"
@@ -78,7 +78,7 @@ static struct db *newsrc_db = NULL;
 static int newsrc_dbopen = 0;
 
 /* must be called after cyrus_init */
-int newsrc_init(char *fname, int myflags __attribute__((unused)))
+int newsrc_init(const char *fname, int myflags __attribute__((unused)))
 {
     char buf[1024];
     int r = 0;
@@ -89,12 +89,13 @@ int newsrc_init(char *fname, int myflags __attribute__((unused)))
     else {
 	char *tofree = NULL;
 
+	if (!fname)
+	    fname = config_getstring(IMAPOPT_NEWSRC_DB_PATH);
+
 	/* create db file name */
 	if (!fname) {
-	    fname = xmalloc(strlen(config_dir)+sizeof(FNAME_NEWSRCDB));
-	    tofree = fname;
-	    strcpy(fname, config_dir);
-	    strcat(fname, FNAME_NEWSRCDB);
+	    tofree = strconcat(config_dir, FNAME_NEWSRCDB, (char *)NULL);
+	    fname = tofree;
 	}
 
 	r = (DB->open)(fname, CYRUSDB_CREATE, &newsrc_db);
@@ -104,7 +105,7 @@ int newsrc_init(char *fname, int myflags __attribute__((unused)))
 	else
 	    newsrc_dbopen = 1;
 
-	if (tofree) free(tofree);
+	free(tofree);
     }
 
     return r;
@@ -293,7 +294,7 @@ int main(int argc, char *argv[])
 	    break;
 
 	case 'f': /* timestamp file */
-	    snprintf(sfile, sizeof(sfile), optarg);
+	    snprintf(sfile, sizeof(sfile), "%s", optarg);
 	    break;
 
 	case 'a': /* authname */
@@ -352,7 +353,7 @@ int main(int argc, char *argv[])
 	else if (!strncmp("381", buf, 3)) {
 	    /* password required */
 	    if (!password)
-		password = getpass("Please enter the password: ");
+		password = cyrus_getpass("Please enter the password: ");
 
 	    if (!password) {
 		fprintf(stderr, "failed to get password\n");
