@@ -104,7 +104,6 @@ struct cyrusdb_backend *config_ptscache_db;
 struct cyrusdb_backend *config_statuscache_db;
 struct cyrusdb_backend *config_userdeny_db;
 
-#define MAX_SESSIONID_SIZE 256
 char session_id_buf[MAX_SESSIONID_SIZE];
 int session_id_time = 0;
 int session_id_count = 0;
@@ -187,7 +186,7 @@ int cyrus_init(const char *alt_config, const char *ident, unsigned flags)
 
     val = config_getstring(IMAPOPT_SUPPRESS_CAPABILITIES);
     if (val)
-	suppressed_capabilities = strarray_split(val, NULL);
+	suppressed_capabilities = strarray_split(val, NULL, 0);
 
     if (!cyrus_init_nodb) {
 	/* lookup the database backends */
@@ -468,7 +467,7 @@ int mysasl_canon_user(sasl_conn_t *conn,
 	sasl_seterror(conn, 0, "buffer overflow while canonicalizing");
 	return SASL_BUFOVER;
     }
-    memcpy(out, user, ulen);
+    memmove(out, user, ulen);
     out[ulen] = '\0';
 
     canonuser = canonify_userid(out, NULL, (int*) context);
@@ -808,6 +807,29 @@ const char *session_id()
     return (const char *)session_id_buf;
 }
 
+/* parse sessionid out of protocol answers */
+void parse_sessionid(const char *str, char *sessionid)
+{
+    char *sp, *ep;
+    int len;
+
+    if ((str) && (sp = strstr(str, "SESSIONID=<")) && (ep = strchr(sp, '>')))
+    {
+	sp += 11;
+	len = ep - sp;
+	if (len < MAX_SESSIONID_SIZE)
+	{
+	    strncpy(sessionid, sp, len);
+	    ep = sessionid + len;
+	    *ep = '\0';
+	}
+	else
+	    strcpy(sessionid, "invalid");
+    }
+    else
+	strcpy(sessionid, "unknown");
+}
+
 int capa_is_disabled(const char *str)
 {
     if (!suppressed_capabilities) return 0;
