@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: imapd.h,v 1.71 2008/03/24 17:09:17 murch Exp $
+ * $Id: imapd.h,v 1.72 2010/01/06 17:01:34 murch Exp $
  */
 
 #ifndef INCLUDED_IMAPD_H
@@ -57,11 +57,8 @@ extern char *imapd_userid;
 /* Authorization state for logged in userid */
 extern struct auth_state *imapd_authstate;
 
-/* Number of messages in currently open mailbox */
-extern int imapd_exists;
-
-/* Is client CONDSTORE-aware? */
-extern int imapd_condstore_client;
+/* Client capabilities (via ENABLE) */
+extern unsigned imapd_client_capa;
 
 /* List of HEADER.FIELDS[.NOT] fetch specifications */
 struct fieldlist {
@@ -84,6 +81,9 @@ struct fetchargs {
     int start_octet;              /* start_octet for partial fetch */
     int octet_count;              /* octet_count for partial fetch, or 0 */
     modseq_t changedsince;        /* changed since modseq, or 0 */
+    int vanished;                 /* report expunges since changedsince */
+    const char *match_seq;
+    const char *match_uid;        /* sequence match data for VANISHED */
 
     bit32 cache_atleast;          /* to do headers we need atleast this
 				   * cache version */
@@ -152,6 +152,7 @@ struct searchsub {
     struct searchargs *sub2;
 };
 
+/* Bitmasks for search flags */
 enum {
     SEARCH_RECENT_SET =         (1<<0),
     SEARCH_RECENT_UNSET	=       (1<<1),
@@ -160,6 +161,14 @@ enum {
 /*    SEARCH_UNCACHEDHEADER =	(1<<4) -- obsolete */
 };
 
+/* Bitmasks for search return options */
+enum {
+    SEARCH_RETURN_MIN =		(1<<0),
+    SEARCH_RETURN_MAX =		(1<<1),
+    SEARCH_RETURN_ALL =		(1<<2),
+    SEARCH_RETURN_COUNT =	(1<<3)
+};
+
 /* Things that may be searched for */
 struct searchargs {
     int flags;
@@ -170,8 +179,8 @@ struct searchargs {
     bit32 system_flags_unset;
     bit32 user_flags_set[MAX_USER_FLAGS/32];
     bit32 user_flags_unset[MAX_USER_FLAGS/32];
-    struct seq_set *sequence;
-    struct seq_set *uidsequence;
+    struct seqset *sequence;
+    struct seqset *uidsequence;
     struct strlist *from;
     struct strlist *to;
     struct strlist *cc;
@@ -185,6 +194,10 @@ struct searchargs {
     modseq_t modseq;
 
     bit32 cache_atleast;
+
+    /* For ESEARCH */
+    const char *tag;
+    int returnopts;
 };
 
 /* Sort criterion */
@@ -205,6 +218,8 @@ enum {
     SORT_ARRIVAL,
     SORT_CC,
     SORT_DATE,
+    SORT_DISPLAYFROM,
+    SORT_DISPLAYTO,
     SORT_FROM,
     SORT_SIZE,
     SORT_SUBJECT,
@@ -229,69 +244,66 @@ enum {
 
 /* Arguments to List functions */
 struct listargs {
-    unsigned opts;		/* LISTEXT options */
+    unsigned cmd;		/* Command variant */
+    unsigned sel;		/* Selection options */
+    unsigned ret;		/* Return options */
     const char *ref;		/* Reference name */
-    const char *pat;		/* Mailbox pattern */
+    struct strlist *pat;	/* Mailbox pattern(s) */
     const char *scan;		/* SCAN content */
     hash_table server_table;	/* for proxying SCAN */
 };
 
-/* Bitmask for list options */
+/* Value for List command variant */
 enum {
-    LIST_LSUB =			(1<<0),
-    LIST_EXT =			(1<<1),
-    LIST_SUBSCRIBED =		(1<<2),
-    LIST_CHILDREN =		(1<<3),
-    LIST_REMOTE =		(1<<4)
+    LIST_CMD_LIST = 0,
+    LIST_CMD_LSUB,
+    LIST_CMD_EXTENDED,
+    LIST_CMD_XLIST,
+};
+
+/* Bitmask for List selection options */
+enum {
+    LIST_SEL_SUBSCRIBED =	(1<<0),
+    LIST_SEL_REMOTE =		(1<<1),
+    LIST_SEL_RECURSIVEMATCH =	(1<<2)
+};
+
+/* Bitmask for List return options */
+enum {
+    LIST_RET_SUBSCRIBED =	(1<<0),
+    LIST_RET_CHILDREN =		(1<<1)
+};
+
+/* Bitmask for List name attributes */
+enum {
+    /* from RFC 3501 */
+    MBOX_ATTRIBUTE_NOINFERIORS =	(1<<0),
+    MBOX_ATTRIBUTE_NOSELECT =		(1<<1),
+    MBOX_ATTRIBUTE_MARKED =		(1<<2),
+    MBOX_ATTRIBUTE_UNMARKED =		(1<<3),
+
+    /* from draft-ietf-imapext-list-extensions-18.txt */
+    MBOX_ATTRIBUTE_NONEXISTENT =	(1<<4),
+    MBOX_ATTRIBUTE_SUBSCRIBED =		(1<<5),
+    MBOX_ATTRIBUTE_REMOTE =		(1<<6),
+    MBOX_ATTRIBUTE_HASCHILDREN =	(1<<7),
+    MBOX_ATTRIBUTE_HASNOCHILDREN =	(1<<8),
+    MBOX_ATTRIBUTE_CHILDINFO_SUBSCRIBED = (1<<9)
+};
+
+/* Bitmask for client capabilities */
+enum {
+    CAPA_CONDSTORE =	(1<<0),
+    CAPA_QRESYNC = 	(1<<1)
+};
+
+/* Bitmask for urlfetch params */
+enum {
+    URLFETCH_BODY =			(1<<0),
+    URLFETCH_BINARY =			(1<<1),
+    URLFETCH_BODYPARTSTRUCTURE =	(1<<2)
 };
 
 extern struct protstream *imapd_out, *imapd_in;
 
-extern void index_closemailbox(struct mailbox *mailbox);
-extern void index_newmailbox(struct mailbox *mailbox, int examine_mode);
-extern void index_operatemailbox(struct mailbox *mailbox);
-extern void index_check(struct mailbox *mailbox, int usinguid,
-			   int checkseen);
-extern void index_check_existing(struct mailbox *mailbox,
-                                 int usinguid, int checkseen);
-extern void index_checkseen(struct mailbox *mailbox, int quiet,
-			       int usinguid, int oldexists);
-
-extern int index_fetch(struct mailbox *mailbox, const char *sequence,
-		       int usinguid, struct fetchargs *fetchargs,
-		       int* fetchedsomething);
-extern int index_store(struct mailbox *mailbox, char *sequence,
-			  int usinguid, struct storeargs *storeargs,
-			  char **flag, int nflags);
-extern int index_search(struct mailbox *mailbox,
-			struct searchargs *searchargs, int usinguid);
-extern int find_thread_algorithm(char *arg);
-extern int index_scan(struct mailbox *mailbox, const char *contents);
-extern int index_sort(struct mailbox *mailbox, struct sortcrit *sortcrit,
-		      struct searchargs *searchargs, int usinguid);
-extern int index_thread(struct mailbox *mailbox, int algorithm,
-			struct searchargs *searchargs, int usinguid);
-extern int index_copy(struct mailbox *mailbox, char *sequence,
-		      int usinguid, char *name, char **copyuidp, int nolink);
-extern int index_status(char *mboxname, char *name, unsigned statusitems);
-
-extern int index_getuids(struct mailbox *mailbox, unsigned lowuid);
-extern int index_getstate(struct mailbox *mailbox);
-extern int index_checkstate(struct mailbox *mailbox, unsigned indexdate,
-			       unsigned seendate);
-
-extern unsigned index_finduid(unsigned uid);
-extern unsigned index_getuid(unsigned msgno);
-
-extern mailbox_decideproc_t index_expungeuidlist;
-
-/* See lib/charset.h for the definition of receiver. */
-extern void index_getsearchtext_single(struct mailbox* mailbox, unsigned msgno,
-                                       index_search_text_receiver_t receiver,
-                                       void* rock);
-
-extern void index_getsearchtext(struct mailbox* mailbox,
-                                index_search_text_receiver_t receiver,
-                                void* rock);
-
 #endif /* INCLUDED_IMAPD_H */
