@@ -63,6 +63,7 @@
 #include "annotate.h" /* for strlist functionality */
 #include "message_guid.h"
 #include "sequence.h"
+#include "strarray.h"
 
 /* Special "sort criteria" to load message-id and references/in-reply-to
  * into msgdata array for threaders that need them.
@@ -145,6 +146,8 @@ typedef struct msgdata {
     char *cc;			/* local-part of first "cc" address */
     char *from;			/* local-part of first "from" address */
     char *to;			/* local-part of first "to" address */
+    char *displayfrom;          /* display-name of first "from" address */
+    char *displayto;            /* display-name of first "to" address */
     char *xsubj;		/* extracted subject text */
     unsigned xsubj_hash;	/* hash of extracted subject text */
     int is_refwd;		/* is message a reply or forward? */
@@ -191,7 +194,7 @@ extern int index_store(struct index_state *state,
 		       char *sequence,
 		       int usinguid,
 		       struct storeargs *storeargs,
-		       char **flag, int nflags);
+		       const strarray_t *flags);
 extern int index_sort(struct index_state *state, struct sortcrit *sortcrit,
 		      struct searchargs *searchargs, int usinguid);
 extern int index_thread(struct index_state *state, int algorithm,
