@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: lmtp_sieve.c,v 1.19 2009/03/31 04:11:18 brong Exp $
  */
 
 #include <config.h>
@@ -225,6 +223,7 @@ static int send_rejection(const char *origid,
     time_t t;
     char datestr[80];
     pid_t sm_pid, p;
+    duplicate_key_t dkey = DUPLICATE_INITIALIZER;
 
     smbuf[0] = "sendmail";
     smbuf[1] = "-i";		/* ignore dots */
@@ -244,10 +243,13 @@ static int send_rejection(const char *origid,
 	     global_outgoing_count++, config_servername);
     
     namebuf = make_sieve_db(mailreceip);
-    duplicate_mark(buf, strlen(buf), namebuf, strlen(namebuf), t, 0);
-    fprintf(sm, "Message-ID: %s\r\n", buf);
 
+    dkey.id = buf;
+    dkey.to = namebuf;
     rfc822date_gen(datestr, sizeof(datestr), t);
+    dkey.date = datestr;
+    duplicate_mark(&dkey, t, 0);
+    fprintf(sm, "Message-ID: %s\r\n", buf);
     fprintf(sm, "Date: %s\r\n", datestr);
 
     fprintf(sm, "X-Sieve: %s\r\n", SIEVE_VERSION);
@@ -277,7 +279,7 @@ static int send_rejection(const char *origid,
 	    "Content-Type: message/disposition-notification\r\n\r\n",
 	    (int) p, config_servername);
     fprintf(sm, "Reporting-UA: %s; Cyrus %s/%s\r\n",
-	    config_servername, CYRUS_VERSION, SIEVE_VERSION);
+	    config_servername, cyrus_version(), SIEVE_VERSION);
     if (origreceip)
 	fprintf(sm, "Original-Recipient: rfc822; %s\r\n", origreceip);
     fprintf(sm, "Final-Recipient: rfc822; %s\r\n", mailreceip);
@@ -316,11 +318,10 @@ static int send_forward(const char *forwardto,
 
     smbuf[0] = "sendmail";
     smbuf[1] = "-i";		/* ignore dots */
+    smbuf[2] = "-f";
     if (return_path && *return_path) {
-	smbuf[2] = "-f";
 	smbuf[3] = return_path;
     } else {
-	smbuf[2] = "-f";
 	smbuf[3] = "<>";
     }
     smbuf[4] = "--";
@@ -368,6 +369,7 @@ static int sieve_redirect(void *ac,
     script_data_t *sd = (script_data_t *) sc;
     message_data_t *m = ((deliver_data_t *) mc)->m;
     char buf[8192], *sievedb = NULL;
+    duplicate_key_t dkey = DUPLICATE_INITIALIZER;
     int res;
 
     /* if we have a msgid, we can track our redirects */
@@ -375,21 +377,26 @@ static int sieve_redirect(void *ac,
 	snprintf(buf, sizeof(buf), "%s-%s", m->id, rc->addr);
 	sievedb = make_sieve_db(sd->username);
 
+	dkey.id = buf;
+	dkey.to = sievedb;
+	dkey.date = ((deliver_data_t *) mc)->m->date;
 	/* ok, let's see if we've redirected this message before */
-	if (duplicate_check(buf, strlen(buf), sievedb, strlen(sievedb))) {
-	    duplicate_log(m->id, sd->username, "redirect");
+	if (duplicate_check(&dkey)) {
+	    duplicate_log(&dkey, "redirect");
 	    return SIEVE_OK;
 	}
     }
 
     if ((res = send_forward(rc->addr, m->return_path, m->data)) == 0) {
 	/* mark this message as redirected */
-	if (sievedb) duplicate_mark(buf, strlen(buf), 
-				    sievedb, strlen(sievedb), time(NULL), 0);
+	if (sievedb) duplicate_mark(&dkey, time(NULL), 0);
 
 	snmp_increment(SIEVE_REDIRECT, 1);
 	syslog(LOG_INFO, "sieve redirected: %s to: %s",
 	       m->id ? m->id : "<nomsgid>", rc->addr);
+	if (config_auditlog)
+	    syslog(LOG_NOTICE, "auditlog: redirect sessionid=<%s> message-id=%s target=<%s>",
+	           session_id(), m->id ? m->id : "<nomsgid>", rc->addr);
 	return SIEVE_OK;
     } else {
 	if (res == -1) {
@@ -414,6 +421,9 @@ static int sieve_discard(void *ac __attribute__((unused)),
     /* ok, we won't file it, but log it */
     syslog(LOG_INFO, "sieve discarded: %s",
 	   md->id ? md->id : "<nomsgid>");
+    if (config_auditlog)
+	syslog(LOG_NOTICE, "auditlog: discard sessionid=<%s> message-id=%s",
+	       session_id(), md->id ? md->id : "<nomsgid>");
 
     return SIEVE_OK;
 }
@@ -438,7 +448,10 @@ static int sieve_reject(void *ac,
     if (strlen(md->return_path) == 0) {
 	syslog(LOG_INFO, "sieve: discarded reject to <> for %s id %s",
 	       sd->username, md->id ? md->id : "<nomsgid>");
-        return SIEVE_OK;
+	if (config_auditlog)
+	    syslog(LOG_NOTICE, "auditlog: discard-reject sessionid=<%s> message-id=%s",
+	           session_id(), md->id ? md->id : "<nomsgid>");
+	return SIEVE_OK;
     }
 
     body = msg_getheader(md, "original-recipient");
@@ -449,6 +462,9 @@ static int sieve_reject(void *ac,
 	snmp_increment(SIEVE_REJECT, 1);
 	syslog(LOG_INFO, "sieve rejected: %s to: %s",
 	       md->id ? md->id : "<nomsgid>", md->return_path);
+	if (config_auditlog)
+	    syslog(LOG_NOTICE, "auditlog: reject sessionid=<%s> message-id=%s target=<%s>",
+	           session_id(), md->id ? md->id : "<nomsgid>", md->return_path);
 	return SIEVE_OK;
     } else {
 	if (res == -1) {
@@ -479,10 +495,10 @@ static int sieve_fileinto(void *ac,
 						   sd->username, namebuf);
     if (!ret) {
 	ret = deliver_mailbox(md->f, mdata->content, mdata->stage, md->size,
-			      fc->imapflags->flag, fc->imapflags->nflags,
+			      fc->imapflags,
 			      (char *) sd->username, sd->authstate, md->id,
 			      sd->username, mdata->notifyheader,
-			      namebuf, quotaoverride, 0);
+			      namebuf, md->date, quotaoverride, 0);
     }
 
     if (!ret) {
@@ -503,7 +519,7 @@ static int sieve_keep(void *ac,
     deliver_data_t *mydata = (deliver_data_t *) mc;
     int ret;
 
-    ret = deliver_local(mydata, kc->imapflags->flag, kc->imapflags->nflags,
+    ret = deliver_local(mydata, kc->imapflags,
 			(char *) sd->username, sd->mailboxname);
 
     if (!ret) {
@@ -542,6 +558,8 @@ static int sieve_notify(void *ac,
     return SIEVE_OK;
 }
 
+static const char hex[] = "0123456789ABCDEF";
+
 static int autorespond(void *ac, 
 		       void *ic __attribute__((unused)), 
 		       void *sc,
@@ -552,14 +570,25 @@ static int autorespond(void *ac,
     script_data_t *sd = (script_data_t *) sc;
     time_t t, now;
     int ret;
+    int i;
+    duplicate_key_t dkey = DUPLICATE_INITIALIZER;
+    char *id;
 
     snmp_increment(SIEVE_VACATION_TOTAL, 1);
 
     now = time(NULL);
 
     /* ok, let's see if we've responded before */
-    t = duplicate_check((char *) arc->hash, SIEVE_HASHLEN, 
-			sd->username, strlen(sd->username));
+    id = xmalloc(SIEVE_HASHLEN*2 + 1);
+    for (i = 0; i < SIEVE_HASHLEN; i++) {
+	id[i*2+0] = hex[arc->hash[i] / 16];
+	id[i*2+1] = hex[arc->hash[i] % 16];
+    }
+    id[SIEVE_HASHLEN*2] = '\0';
+    dkey.id = id;
+    dkey.to = sd->username;
+    dkey.date = "";  /* no date on these, ID is custom */
+    t = duplicate_check(&dkey);
     if (t) {
 	if (now >= t) {
 	    /* yay, we can respond again! */
@@ -573,11 +602,11 @@ static int autorespond(void *ac,
     }
 
     if (ret == SIEVE_OK) {
-	duplicate_mark((char *) arc->hash, SIEVE_HASHLEN, 
-		       sd->username, strlen(sd->username), 
-		       now + arc->days * (24 * 60 * 60), 0);
+	duplicate_mark(&dkey, now + arc->days * (24 * 60 * 60), 0);
     }
 
+    free(id);
+
     return ret;
 }
 
@@ -595,6 +624,7 @@ static int send_response(void *ac,
     sieve_send_response_context_t *src = (sieve_send_response_context_t *) ac;
     message_data_t *md = ((deliver_data_t *) mc)->m;
     script_data_t *sdata = (script_data_t *) sc;
+    duplicate_key_t dkey = DUPLICATE_INITIALIZER;
 
     smbuf[0] = "sendmail";
     smbuf[1] = "-i";		/* ignore dots */
@@ -655,8 +685,10 @@ static int send_response(void *ac,
     if (sm_stat == 0) { /* sendmail exit value */
 	sievedb = make_sieve_db(sdata->username);
 
-	duplicate_mark(outmsgid, strlen(outmsgid), 
-		       sievedb, strlen(sievedb), t, 0);
+	dkey.id = outmsgid;
+	dkey.to = sievedb;
+	dkey.date = ((deliver_data_t *) mc)->m->date;
+	duplicate_mark(&dkey, t, 0);
 
 	snmp_increment(SIEVE_VACATION_REPLIED, 1);
 
@@ -675,10 +707,6 @@ sieve_vacation_t vacation = {
     &send_response,		/* send_response() */
 };
 
-/* imapflags support */
-static char *markflags[] = { "\\flagged" };
-static sieve_imapflags_t mark = { markflags, 1 };
-
 static int sieve_parse_error_handler(int lineno, const char *msg, 
 				     void *ic __attribute__((unused)),
 				     void *sc)
@@ -708,6 +736,10 @@ sieve_interp_t *setup_sieve(void)
 {
     sieve_interp_t *interp = NULL;
     int res;
+    static strarray_t mark = STRARRAY_INITIALIZER;
+
+    if (!mark.count)
+	strarray_append(&mark, "\\flagged");
 
     sieve_usehomedir = config_getswitch(IMAPOPT_SIEVEUSEHOMEDIR);
     if (!sieve_usehomedir) {
@@ -867,6 +899,7 @@ int run_sieve(const char *user, const char *domain, const char *mailbox,
     char userbuf[MAX_MAILBOX_BUFFER] = "";
     char authuserbuf[MAX_MAILBOX_BUFFER];
     int r = 0;
+    duplicate_key_t dkey = DUPLICATE_INITIALIZER;
 
     if (!user) {
 	/* shared mailbox, check for annotation */
@@ -923,8 +956,10 @@ int run_sieve(const char *user, const char *domain, const char *mailbox,
 		 domain ? domain : "");
 	sdb = make_sieve_db(namebuf);
 		
-	duplicate_mark(msgdata->m->id, strlen(msgdata->m->id), 
-		       sdb, strlen(sdb), time(NULL), 0);
+	dkey.id = msgdata->m->id;
+	dkey.to = sdb;
+	dkey.date = msgdata->m->date;
+	duplicate_mark(&dkey, time(NULL), 0);
     }
 		
     /* free everything */
