@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: lmtpd.c,v 1.167 2010/05/25 20:59:19 wescraig Exp $
  */
 
 #include <config.h>
@@ -90,6 +88,7 @@
 #include "statuscache.h"
 #include "telemetry.h"
 #include "tls.h"
+#include "userdeny.h"
 #include "util.h"
 #include "version.h"
 #include "xmalloc.h"
@@ -226,6 +225,10 @@ int service_init(int argc __attribute__((unused)),
 	quotadb_init(0);
 	quotadb_open(NULL);
 
+	/* open the user deny db */
+	denydb_init(0);
+	denydb_open(NULL);
+
 	/* Initialize the annotatemore db (for sieve on shared mailboxes) */
 	annotatemore_init(0, NULL, NULL);
 	annotatemore_open(NULL);
@@ -336,8 +339,7 @@ void service_abort(int error)
     shut_down(error);
 }
 
-static void
-usage()
+static void usage()
 {
     fprintf(stderr, "421-4.3.0 usage: lmtpd [-C <alt_config>] [-a]\r\n");
     fprintf(stderr, "421 4.3.0 %s\n", cyrus_version());
@@ -446,6 +448,8 @@ static int mlookup(const char *name, char **server, char **aclp, void *tid)
 	    fatal("error communicating with MUPDATE server", EC_TEMPFAIL);
 	}
 
+	if (mailboxdata->t == RESERVE) return IMAP_MAILBOX_RESERVED;
+
 	if (aclp) *aclp = (char *) mailboxdata->acl;
 	if (server) *server = (char *) mailboxdata->server;
 	c = strchr(*server, '!');
@@ -460,6 +464,8 @@ static int mlookup(const char *name, char **server, char **aclp, void *tid)
 	    r = mboxlist_lookup(name, &mbentry, tid);
 	}
 	if (r) return r;
+	if (mbentry.mbtype & MBTYPE_MOVING)  return IMAP_MAILBOX_MOVED;
+	if (mbentry.mbtype & MBTYPE_DELETED) return IMAP_MAILBOX_NONEXISTENT;
 
 	if (aclp) *aclp = mbentry.acl;
 	if (server) {
@@ -475,7 +481,7 @@ static int mlookup(const char *name, char **server, char **aclp, void *tid)
     return r;
 }
 
-/* places msg in mailbox mailboxname.  
+/* places msg in mailbox mailboxname.
  * if you wish to use single instance store, pass stage as non-NULL
  * if you want to deliver message regardless of duplicates, pass id as NULL
  * if you want to notify, pass user
@@ -486,8 +492,7 @@ int deliver_mailbox(FILE *f,
 		    struct message_content *content,
 		    struct stagemsg *stage,
 		    unsigned size,
-		    char **flag,
-		    int nflags,
+		    const strarray_t *flags,
 		    char *authuser,
 		    struct auth_state *authstate,
 		    char *id,
@@ -529,7 +534,7 @@ int deliver_mailbox(FILE *f,
 
     if (!r) {
 	r = append_fromstage(&as, &content->body, stage, 0,
-			     (const char **) flag, nflags, !singleinstance);
+			     flags, !singleinstance);
 
 	if (r) {
 	    append_abort(&as);
@@ -563,7 +568,7 @@ int deliver_mailbox(FILE *f,
 	    if (strlen(mailboxname) >= inboxlen &&
 		!strncmp(mailboxname, inbox, inboxlen) &&
 		(!mailboxname[inboxlen] || mailboxname[inboxlen] == '.')) {
-		strlcpy(inbox, "INBOX", sizeof(inbox)); 
+		strlcpy(inbox, "INBOX", sizeof(inbox));
 		strlcat(inbox, mailboxname+inboxlen, sizeof(inbox));
 		notify_mailbox = inbox;
 	    }
@@ -646,7 +651,7 @@ void deliver_remote(message_data_t *msgdata,
 	    int j = rc->rcpt_num;
 	    switch (status[j]) {
 	    case s_wait:
-		/* hmmm, if something fails we'll want to try an 
+		/* hmmm, if something fails we'll want to try an
 		   error delivery */
 		if (lt->rcpt[i].result != RCPT_GOOD) {
 		    status[j] = s_err;
@@ -675,7 +680,7 @@ void deliver_remote(message_data_t *msgdata,
     }
 }
 
-int deliver_local(deliver_data_t *mydata, char **flag, int nflags,
+int deliver_local(deliver_data_t *mydata, const strarray_t *flags,
 		  const char *username, const char *mailboxname)
 {
     char namebuf[MAX_MAILBOX_BUFFER] = "", *tail;
@@ -689,7 +694,7 @@ int deliver_local(deliver_data_t *mydata, char **flag, int nflags,
 	strlcat(namebuf, mailboxname, sizeof(namebuf));
 
 	return deliver_mailbox(md->f, mydata->content, mydata->stage,
-			       md->size, flag, nflags,
+			       md->size, flags,
 			       mydata->authuser, mydata->authstate, md->id,
 			       NULL, mydata->notifyheader,
 			       namebuf, md->date, quotaoverride, 0);
@@ -709,7 +714,7 @@ int deliver_local(deliver_data_t *mydata, char **flag, int nflags,
 	    strlcat(namebuf, mailboxname, sizeof(namebuf));
 
 	    ret2 = deliver_mailbox(md->f, mydata->content, mydata->stage,
-				   md->size, flag, nflags,
+				   md->size, flags,
 				   mydata->authuser, mydata->authstate, md->id,
 				   username, mydata->notifyheader,
 				   namebuf, md->date, quotaoverride, 0);
@@ -719,7 +724,7 @@ int deliver_local(deliver_data_t *mydata, char **flag, int nflags,
 	    fuzzy_match(namebuf)) {
 	    /* try delivery to a fuzzy matched mailbox */
 	    ret2 = deliver_mailbox(md->f, mydata->content, mydata->stage,
-				   md->size, flag, nflags,
+				   md->size, flags,
 				   mydata->authuser, mydata->authstate, md->id,
 				   username, mydata->notifyheader,
 				   namebuf, md->date, quotaoverride, 0);
@@ -731,7 +736,7 @@ int deliver_local(deliver_data_t *mydata, char **flag, int nflags,
 	    *tail = '\0';
 
 	    ret = deliver_mailbox(md->f, mydata->content, mydata->stage,
-				  md->size, flag, nflags,
+				  md->size, flags,
 				  (char *) username, authstate, md->id,
 				  username, mydata->notifyheader,
 				  namebuf, md->date, quotaoverride, 1);
@@ -752,7 +757,7 @@ int deliver(message_data_t *msgdata, char *authuser,
     struct message_content content = { NULL, 0, NULL };
     char *notifyheader;
     deliver_data_t mydata;
-    
+
     assert(msgdata);
     nrcpts = msg_getnumrcpt(msgdata);
     assert(nrcpts);
@@ -770,7 +775,7 @@ int deliver(message_data_t *msgdata, char *authuser,
     mydata.namespace = &lmtpd_namespace;
     mydata.authuser = authuser;
     mydata.authstate = authstate;
-    
+
     /* loop through each recipient, attempting delivery for each */
     for (n = 0; n < nrcpts; n++) {
 	char namebuf[MAX_MAILBOX_BUFFER] = "", *server;
@@ -820,7 +825,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 #endif
 
 	    if (r) {
-		r = deliver_local(&mydata, NULL, 0, userbuf, mailbox);
+		r = deliver_local(&mydata, NULL, userbuf, mailbox);
 	    }
 	}
 
@@ -839,7 +844,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	while (d) {
 	    struct dest *nextd = d->next;
 	    struct rcpt *rc = d->to;
-   
+
 	    while (rc) {
 		struct rcpt *nextrc = rc->next;
 		free(rc);
@@ -857,7 +862,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	    case s_err:
 	    case s_done:
 		/* yikes, we haven't implemented sieve ! */
-		syslog(LOG_CRIT, 
+		syslog(LOG_CRIT,
 		       "sieve states reached, but we don't implement sieve");
 		abort();
 	    break;
@@ -881,7 +886,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	    assert(status[n] == done || status[n] == s_done);
 	}
     }
-   
+
     /* cleanup */
     free(status);
     if (content.base) map_free(&content.base, &content.len);
@@ -899,7 +904,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 void fatal(const char* s, int code)
 {
     static int recurse_code = 0;
-    
+
     if(recurse_code) {
 	/* We were called recursively. Just give up */
 	snmp_increment(ACTIVE_CONNECTIONS, -1);
@@ -913,7 +918,7 @@ void fatal(const char* s, int code)
     if (stage) append_removestage(stage);
 
     syslog(LOG_ERR, "FATAL: %s", s);
-    
+
     /* shouldn't return */
     shut_down(code);
 
@@ -956,6 +961,9 @@ void shut_down(int code)
 	quotadb_close();
 	quotadb_done();
 
+	denydb_close();
+	denydb_done();
+
 	annotatemore_close();
 	annotatemore_done();
 
@@ -1052,6 +1060,18 @@ static int verify_user(const char *user, const char *domain, char *mailbox,
 	}
     }
 
+    if (!r && user) {
+	char msg[MAX_MAILBOX_PATH+1];
+
+	if (domain) {
+	    snprintf(namebuf, sizeof(namebuf), "%s@%s", user, domain);
+	    user = namebuf;
+	}
+
+	if (userdeny(user, config_ident, msg, sizeof(msg)))
+	    return IMAP_MAILBOX_DISABLED;
+    }
+
     if (r) syslog(LOG_DEBUG, "verify_user(%s) failed: %s", namebuf,
 		  error_message(r));
 
@@ -1082,7 +1102,7 @@ char *generate_notify(message_data_t *m)
 		    ret = xrealloc(ret, len += 1024);
 		}
 		pos += sprintf(ret + pos, "%s: ", h);
-		
+
 		/* put the header body.
 		   xxx it would be nice to linewrap.*/
 		/* need: length + '\n' + '\0' */
