@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: lmtpengine.c,v 1.132 2010/01/06 17:01:35 murch Exp $
  */
 
 #include <config.h>
@@ -222,6 +220,15 @@ static void send_lmtp_error(struct protstream *pout, int r)
 	prot_printf(pout, "451 4.2.1 Mailbox Moved\r\n");
 	break;
 
+    case IMAP_MAILBOX_RESERVED:
+	prot_printf(pout, "451 4.2.1 Mailbox Reserved\r\n");
+	break;
+
+    case IMAP_MAILBOX_DISABLED:
+	prot_printf(pout,
+		    "450 4.2.1 Mailbox disabled, not accepting messages\r\n");
+	break;
+
     case IMAP_MESSAGE_CONTAINSNULL:
 	prot_printf(pout, "554 5.6.0 Message contains NUL characters\r\n");
 	break;
@@ -530,7 +537,7 @@ static char *parseaddr(char *s)
 	    while (Uisalnum(*p) || *p == '.' || *p == '-') p++;
 	}
     }
-    
+
     if (*p++ != '>') return 0;
     if (*p && *p != ' ') return 0;
     len = p - s;
@@ -542,7 +549,7 @@ static char *parseaddr(char *s)
 }
 
 /* clean off the <> from the return path */
-void clean_retpath(char *rpath)
+static void clean_retpath(char *rpath)
 {
     int sl;
 
@@ -563,9 +570,7 @@ void clean_retpath(char *rpath)
  * from string pointed to by 'buf'.  Does not handle continuation header
  * lines.
  */
-void
-clean822space(buf)
-char *buf;
+static void clean822space(char* buf)
 {
     char *from=buf, *to=buf;
     int c;
@@ -1233,7 +1238,7 @@ void lmtpmode(struct lmtp_func *func,
 				  strlcpy(hbuf, "[unknown]", sizeof(hbuf));
 			  }
 			  else
-			      strlcpy(hbuf, "[unix socket]", sizeof(hbuf));		  
+			      strlcpy(hbuf, "[unix socket]", sizeof(hbuf));
 			  syslog(LOG_ERR, "badlogin: %s %s %s",
 				 hbuf, mech, sasl_errdetail(cd.conn));
 		  
@@ -1714,6 +1719,13 @@ static int revconvert_lmtp(const char *code)
 	else if (code[4] == '4' && code [6] == '4') {
 	    return IMAP_SERVER_UNAVAILABLE;
 	}
+	else if (code[4] == '4' && code[6] == '2') {
+	    if (code[8] == '1') {
+		return IMAP_MAILBOX_MOVED;
+	    } else {
+		return IMAP_MAILBOX_BADFORMAT;
+	    }
+	}
 	else {
 	    return IMAP_IOERROR;
 	}
