@@ -39,13 +39,14 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: mbdump.h,v 1.6 2008/03/24 17:09:17 murch Exp $
+ * $Id: mbdump.h,v 1.7 2010/01/06 17:01:36 murch Exp $
  */
 
 #ifndef INCLUDED_MBDUMP_H
 #define INCLUDED_MBDUMP_H
 
 #include "prot.h"
+#include "mailbox.h"
 
 /* if tag is non-null, we assume that we are a server sending to the
  * client, and:
@@ -58,13 +59,11 @@
  * (note that this assumes server LITERAL+ support, but we don't care since
  * this is a Cyrus-only extention)
  */
-extern int dump_mailbox(const char *tag, const char *mbname,
-			const char *mbpath, const char *metapath,
-			const char *mbacl, int uid_start,
+extern int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
+			int oldversion,
 			struct protstream *pin, struct protstream *pout,
 			struct auth_state *auth_state);
-extern int undump_mailbox(const char *mbname, const char *mbpath,
-			  const char *metapath, const char *mbacl,
+extern int undump_mailbox(const char *mbname,
 			  struct protstream *pin, struct protstream *pout,
 			  struct auth_state *auth_state);
 
