@@ -582,7 +582,7 @@ int mboxlist_createmailbox_full(const char *name, int mbtype,
 	/* 2. verify ACL's to best of ability (CRASH: abort) */
 	r = mboxlist_mycreatemailboxcheck(name, mbtype, partition, isadmin, 
 					  userid, auth_state, 
-					  &acl, &newpartition, 1, localonly,
+					  &acl, &newpartition, 0, localonly,
 					  forceuser, NULL);
 	if (r) goto done;
     }
@@ -1001,7 +1001,6 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
     const char *root = NULL;
     char *newpartition = NULL;
     char *mboxent = NULL;
-    char *p;
     mupdate_handle *mupdate_h = NULL;
 
     /* 1. open mailbox */
@@ -1010,12 +1009,23 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 
     myrights = cyrus_acl_myrights(auth_state, oldmailbox->acl);
 
-    /* 2. verify acls */
+    /* check the ACLs up-front */
+    if (!isadmin) {
+	if (!(myrights & ACL_DELETEMBOX)) {
+	    r = (myrights & ACL_LOOKUP) ?
+		IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
+	    goto done;
+	}
+    }
+
+    /* 2. verify valid move */
     /* XXX - handle remote mailbox */
+
+    /* special case: same mailbox, must be a partition move */
     if (!strcmp(oldname, newname)) {
 	char *oldpath = mailbox_datapath(oldmailbox);
 
-	/* Attempt to move mailbox across partition */
+	/* Only admin can move mailboxes between partitions */
 	if (!isadmin) {
 	    r = IMAP_PERMISSION_DENIED;
 	    goto done;
@@ -1027,7 +1037,9 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 	    goto done;
 	}
 
+	/* let mupdate code below know it was a partition move */
 	partitionmove = 1;
+
 	/* this is OK because it uses a different static buffer */
 	root = config_partitiondir(partition);
 	if (!root) {
@@ -1040,39 +1052,7 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 	    r = IMAP_MAILBOX_EXISTS;
 	    goto done;
 	}
-    } else if ((p = mboxname_isusermailbox(oldname, 1))) {
-	if (!strncmp(p, userid, config_virtdomains ? strcspn(userid, "@") :
-		     strlen(userid))) {
-	    /* Special case of renaming inbox */
-	    if (!(myrights & ACL_DELETEMBOX)) {
-		r = IMAP_PERMISSION_DENIED;
-		goto done;
-	    }
-	    isusermbox = 1;
-	} else if ((config_getswitch(IMAPOPT_ALLOWUSERMOVES) &&
-		    mboxname_isusermailbox(newname, 1)) ||
-		   mboxname_isdeletedmailbox(newname)) {
-	    /* Special case of renaming a user */
-	    if (!(myrights & ACL_DELETEMBOX) && !isadmin) {
-		r = (myrights & ACL_LOOKUP) ?
-		    IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
-		goto done;
-	    }
-	} else {
-	    /* Even admins can only rename an INBOX to another INBOX */
-	    r = IMAP_MAILBOX_NOTSUPPORTED;
-	    goto done;
-	}
-    } else {
-	if (!(myrights & ACL_DELETEMBOX) && !isadmin) {
-	    r = (myrights & ACL_LOOKUP) ?
-		IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
-	    goto done;
-	}
-    }
 
-    /* Check ability to create new mailbox */
-    if (partitionmove) {
 	/* NOTE: this is a rename to the same mailbox name on a
 	 * different partition.  This is a pretty filthy hack,
 	 * which should be handled by having four totally different
@@ -1086,50 +1066,64 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 	r = DB->store(mbdb, newname, strlen(newname), 
 		      mboxent, strlen(mboxent), &tid);
 	if (r) goto done;
+
+	/* skip ahead to the commit */
 	goto dbdone;
     }
-    else {
-	if (mboxname_isusermailbox(newname, 1)) {
-	    if ((config_getswitch(IMAPOPT_ALLOWUSERMOVES) &&
-		 mboxname_isusermailbox(oldname, 1)) ||
-		mboxname_isdeletedmailbox(oldname)) {
-		if (!isadmin) {
-		    /* Only admins can rename users (INBOX to INBOX) */
-		    r = IMAP_MAILBOX_NOTSUPPORTED;
-		    goto done;
-		}
-	    } else {
-		/* Even admins can't rename to user's inboxes */
+
+    /* RENAME of some user's INBOX */
+    if (mboxname_isusermailbox(oldname, 1)) {
+	if (mboxname_isdeletedmailbox(newname)) {
+	    /* delete user is OK */
+	}
+	else if (mboxname_isusermailbox(newname, 1)) {
+	    /* user rename is depends on config */
+	    if (!config_getswitch(IMAPOPT_ALLOWUSERMOVES)) {
 		r = IMAP_MAILBOX_NOTSUPPORTED;
 		goto done;
 	    }
 	}
-	r = mboxlist_mycreatemailboxcheck(newname, 0, partition, isadmin, 
-					  userid, auth_state, NULL, 
-					  &newpartition, 1, 0, forceuser, NULL);
-	if (r) goto done;
-
-	if (!newpartition) newpartition = xstrdup(config_defpartition);
+	else if (mboxname_userownsmailbox(userid, oldname) &&
+		 mboxname_userownsmailbox(userid, newname)) {
+	    /* Special case of renaming inbox */
+	    isusermbox = 1;
+	}
+	else {
+	    /* Everything else is bogus */
+	    r = IMAP_MAILBOX_NOTSUPPORTED;
+	    goto done;
+	}
     }
 
+    r = mboxlist_mycreatemailboxcheck(newname, 0, partition, isadmin, 
+				      userid, auth_state, NULL, 
+				      &newpartition, 1, 0, forceuser, NULL);
+    if (r) goto done;
+
+    if (!newpartition) newpartition = xstrdup(config_defpartition);
+
     /* Rename the actual mailbox */
+    syslog(LOG_INFO, "Rename: %s -> %s", oldname, newname);
+
     r = mailbox_rename_copy(oldmailbox, newname, newpartition,
 			    isusermbox ? userid : NULL, ignorequota,
 			    &newmailbox);
     if (r) goto done;
 
-    /* 7a. create new entry */
+    /* create new entry */
     mboxent = mboxlist_makeentry(newmailbox->mbtype, newmailbox->part,
 				 newmailbox->acl);
     do {
-	/* 7b. put it into the db */
-	r = DB->store(mbdb, newname, strlen(newname), 
-		      mboxent, strlen(mboxent), &tid);
+	r = 0;
 
-	if (!r && !isusermbox) {
-	    /* 4. Delete entry from berkeley db */
+	/* delete the old entry */
+	if (!isusermbox)
 	    r = DB->delete(mbdb, oldname, strlen(oldname), &tid, 0);
-	}
+
+	/* create a new entry */
+	if (!r)
+	    r = DB->store(mbdb, newname, strlen(newname), 
+			  mboxent, strlen(mboxent), &tid);
 
 	switch (r) {
 	case 0: /* success */
@@ -1182,6 +1176,9 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
     }
 
  done: /* Commit or cleanup */
+    if (!r && newmailbox)
+	r = mailbox_commit(newmailbox);
+
     if (r) {
 	/* XXX - rollback DB changes if it was an mupdate failure */
 	if (newmailbox) mailbox_delete(&newmailbox);
@@ -1190,8 +1187,8 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 	mailbox_close(&oldmailbox);
     } else {
 	if (newmailbox) {
-	    mailbox_close(&newmailbox);
 	    mailbox_rename_cleanup(&oldmailbox, isusermbox);
+	    mailbox_close(&newmailbox);
 	}
 	else if (partitionmove) {
 	    char *oldpartition = xstrdup(oldmailbox->part);
@@ -1684,13 +1681,10 @@ static int check_name(struct find_rock *rock,
 		      const char *base, int len)
 {
     if (rock->prev) {
-	/* check if name is previous to what's already been sent.
-	 * NOTE: must be a < 0, not <=0.  It's tempting not to send
-	 * duplicates, but then you can't find if something has
-	 * children in the LIST "" % case, because the matchlen will
-	 * be the same for the children too.  So the callee must
-	 * suppress duplicates after it's checked if it's a child. */
-	if (bsearch_ncompare(base, len, rock->prev, rock->prevlen) < 0)
+	/* check if name is not a substring of the last sent.  Note,
+	 * you actually have to check 1 character PAST the length,
+	 * so you don't forget to tell about subfolders. */
+	if (len <= rock->prevlen && !memcmp(base, rock->prev, len+1))
 	    return 0;
 	free(rock->prev);
     }
