@@ -73,6 +73,7 @@
 #include "xmalloc.h"
 #include "xstrlcpy.h"
 #include "xstrlcat.h"
+#include "strarray.h"
 #include "global.h"
 #include "retry.h"
 
@@ -84,86 +85,6 @@ struct msg {
     int encode;
 };
 
-/* cyrus.cache file item buffer */
-struct ibuf {
-    char *start, *end, *last;
-};
-    
-/*
- * Parsed form of a body-part
- */
-struct body {
-    /* Content-* header information */
-    char *type;
-    char *subtype;
-    struct param *params;
-    char *id;
-    char *description;
-    char *encoding;
-    char *md5;
-    char *disposition;
-    struct param *disposition_params;
-    struct param *language;
-    char *location;
-
-    /* Location/size information */
-    long header_offset;
-    long header_size;
-    long header_lines;
-    long content_offset;
-    long content_size;
-    long content_lines;
-    long boundary_size;		/* Size of terminating boundary */
-    long boundary_lines;
-
-    int numparts;		/* For multipart types */
-    struct body *subpart;	/* For message/rfc822 and multipart types */
-
-    /*
-     * Other header information.
-     * Only meaningful for body-parts at top level or
-     * enclosed in message/rfc-822
-     */
-    char *date;
-    char *subject;
-    struct address *from;
-    struct address *sender;
-    struct address *reply_to;
-    struct address *to;
-    struct address *cc;
-    struct address *bcc;
-    char *in_reply_to;
-    char *message_id;
-    char *received_date;
-
-    /*
-     * Cached headers.  Only filled in at top-level
-     */
-    struct ibuf cacheheaders;
-
-    /*
-     * decoded body.  Filled in as needed.
-     */
-    char *decoded_body;
-
-    /* Message GUID. Only filled in at top level */
-    struct message_guid guid;
-};
-
-/* List of Content-type parameters */
-struct param {
-    struct param *next;
-    char *attribute;
-    char *value;
-};
-
-/* List of pending multipart boundaries */
-struct boundary {
-    char **id;
-    int count;
-    int alloc;
-};
-
 /* (draft standard) MIME tspecials */
 #define TSPECIALS "()<>@,;:\\\"/[]?="
 
@@ -173,12 +94,12 @@ struct boundary {
 static int message_parse_body P((struct msg *msg,
 				 struct body *body,
 				 char *defaultContentType,
-				 struct boundary *boundaries));
+				 strarray_t *boundaries));
 
 static int message_parse_headers P((struct msg *msg,
 				    struct body *body,
 				    char *defaultContentType,
-				    struct boundary *boundaries));
+				    strarray_t *boundaries));
 static void message_parse_address P((char *hdr, struct address **addrp));
 static void message_parse_encoding P((char *hdr, char **hdrp));
 static void message_parse_charset P((struct body *body, int *encoding, int *charset));
@@ -194,14 +115,14 @@ static void message_parse_received_date P((char *hdr, char **hdrp));
 
 static void message_parse_multipart P((struct msg *msg,
 				       struct body *body,
-				       struct boundary *boundaries));
+				       strarray_t *boundaries));
 static void message_parse_content P((struct msg *msg,
 				     struct body *body,
-				     struct boundary *boundaries));
+				     strarray_t *boundaries));
 
 static char *message_getline P((char *s, unsigned n, struct msg *msg));
 static int message_pendingboundary P((const char *s, int len,
-				      char **boundaries, int *boundaryct));
+				      strarray_t *boundaries));
 
 static void message_write_envelope P((struct ibuf *ibuf, struct body *body));
 static void message_write_body P((struct ibuf *ibuf, struct body *body,
@@ -442,7 +363,9 @@ int message_parse_binary_file(FILE *infile, struct body **body)
 
     if (!*body) *body = (struct body *) xmalloc(sizeof(struct body));
     message_parse_body(&msg, *body,
-		       DEFAULT_CONTENT_TYPE, (struct boundary *)0);
+		       DEFAULT_CONTENT_TYPE, (strarray_t *)0);
+
+    message_guid_generate(&(*body)->guid, msg.base, msg.len);
 
     lseek(fd, 0L, SEEK_SET);
     n = retry_write(fd, msg.base, msg.len);
@@ -472,7 +395,7 @@ int message_parse_mapped(const char *msg_base, unsigned long msg_len,
     msg.encode = 0;
 
     message_parse_body(&msg, body,
-		       DEFAULT_CONTENT_TYPE, (struct boundary *)0);
+		       DEFAULT_CONTENT_TYPE, (strarray_t *)0);
 
     message_guid_generate(&body->guid, msg_base, msg_len);
 
@@ -602,18 +525,16 @@ message_parse_body(msg, body, defaultContentType, boundaries)
 struct msg *msg;
 struct body *body;
 char *defaultContentType;
-struct boundary *boundaries;
+strarray_t *boundaries;
 {
-    struct boundary newboundaries;
+    strarray_t newboundaries = STRARRAY_INITIALIZER;
     int sawboundary;
 
     memset(body, 0, sizeof(struct body));
-    newboundaries.id = 0;
 
     /* No passed-in boundary structure, create a new, empty one */
     if (!boundaries) {
 	boundaries = &newboundaries;
-	boundaries->alloc = boundaries->count = 0;
 	/* We're at top-level--set up to store cached headers */
 	message_ibuf_init(&body->cacheheaders);
     }
@@ -657,7 +578,7 @@ struct boundary *boundaries;
     }
 
     /* Free up boundary storage if necessary */
-    if (newboundaries.id) free(newboundaries.id);
+    strarray_fini(&newboundaries);
 
     return 0;
 }
@@ -672,7 +593,7 @@ message_parse_headers(msg, body, defaultContentType,
 struct msg *msg;
 struct body *body;
 char *defaultContentType;
-struct boundary *boundaries;
+strarray_t *boundaries;
 {
     static int alloced = 0;
     static char *headers;
@@ -701,7 +622,7 @@ struct boundary *boundaries;
 	len = strlen(next);
 
 	if (next[-1] == '\n' && *next == '-' &&
-	    message_pendingboundary(next, len, boundaries->id, &boundaries->count)) {
+	    message_pendingboundary(next, len, boundaries)) {
 	    body->boundary_size = len;
 	    body->boundary_lines++;
 	    if (next - 1 > headers) {
@@ -1811,12 +1732,13 @@ static void
 message_parse_multipart(msg, body, boundaries)
 struct msg *msg;
 struct body *body;
-struct boundary *boundaries;
+strarray_t *boundaries;
 {
     struct body preamble, epilogue;
     struct param *boundary;
     char *defaultContentType = DEFAULT_CONTENT_TYPE;
     int i, depth;
+    int limit = config_getint(IMAPOPT_BOUNDARY_LIMIT);
 
     memset(&preamble, 0, sizeof(struct body));
     memset(&epilogue, 0, sizeof(struct body));
@@ -1836,22 +1758,16 @@ struct boundary *boundaries;
 	return;
     }
 
-    /* Expand boundaries array if necessary */
-    if (boundaries->count == boundaries->alloc) {
-	boundaries->alloc += 20;
-	boundaries->id = (char **)xrealloc((char *)boundaries->id,
-					   boundaries->alloc * sizeof(char *));
-    }
-    
     /* Add the new boundary id */
-    boundaries->id[boundaries->count++] = boundary->value;
+    strarray_append(boundaries, boundary->value);
     depth = boundaries->count;
 
     /* Parse preamble */
     message_parse_content(msg, &preamble, boundaries);
 
     /* Parse the component body-parts */
-    while (boundaries->count == depth) {
+    while (boundaries->count == depth &&
+	    (limit == 0 ? 1 : boundaries->count < limit)) {
 	body->subpart = (struct body *)xrealloc((char *)body->subpart,
 				 (body->numparts+1)*sizeof(struct body));
 	message_parse_body(msg, &body->subpart[body->numparts++],
@@ -1912,6 +1828,11 @@ struct boundary *boundaries;
      */
     body->boundary_size += epilogue.boundary_size;
     body->boundary_lines += epilogue.boundary_lines;
+
+    /* check if we've hit a limit and flag it */
+    if (limit && depth == limit) {
+	syslog(LOG_ERR, "ERROR: mime boundary limit %i exceeded, not parsing anymore", limit);
+    }
 }
 
 /*
@@ -1921,7 +1842,7 @@ static void
 message_parse_content(msg, body, boundaries)
 struct msg *msg;
 struct body *body;
-struct boundary *boundaries;
+strarray_t *boundaries;
 {
     const char *line, *endline;
     unsigned long s_offset = msg->offset;
@@ -1945,7 +1866,7 @@ struct boundary *boundaries;
 	msg->offset += len;
 
 	if (line[0] == '-' && line[1] == '-' &&
-	    message_pendingboundary(line, len, boundaries->id, &boundaries->count)) {
+	    message_pendingboundary(line, len, boundaries)) {
 	    body->boundary_size = len;
 	    body->boundary_lines++;
 	    if (body->content_lines) {
@@ -2068,11 +1989,10 @@ struct msg *msg;
  * If we hit a terminating boundary, the integer pointed to by
  * 'boundaryct' is modified appropriately.
  */
-static int message_pendingboundary(s, slen, boundaries, boundaryct)
+static int message_pendingboundary(s, slen, boundaries)
 const char *s;
 int slen;
-char **boundaries;
-int *boundaryct;
+strarray_t *boundaries;
 {
     int i, len;
     int rfc2046_strict = config_getswitch(IMAPOPT_RFC2046_STRICT);
@@ -2085,15 +2005,15 @@ int *boundaryct;
     bbase = s + 2;
     blen = slen - 2;
 
-    for (i = 0; i < *boundaryct; ++i) {
-	len = strlen(boundaries[i]);
+    for (i = 0; i < boundaries->count ; ++i) {
+	len = strlen(boundaries->data[i]);
 	/* basic sanity check and overflow protection */
 	if (blen < len) continue;
 
-	if (!strncmp(bbase, boundaries[i], len)) {
+	if (!strncmp(bbase, boundaries->data[i], len)) {
 	    /* trailing '--', it's the end of this part */
 	    if (blen >= len+2 && bbase[len] == '-' && bbase[len+1] == '-')
-		*boundaryct = i;
+		strarray_truncate(boundaries, i);
 	    else if (!rfc2046_strict && blen > len+1 &&
 		     bbase[len] && !Uisspace(bbase[len])) {
 		/* Allow substring matches in the boundary.
