@@ -1,6 +1,6 @@
 /* message.h -- Message parsing
  *
- * Copyright (c) 1994-2008 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1994-2010 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: message.h,v 1.12 2009/11/19 21:52:55 murch Exp $
+ * $Id: message.h,v 1.13 2010/01/06 17:01:37 murch Exp $
  */
 
 #ifndef INCLUDED_MESSAGE_H
@@ -58,13 +58,88 @@
 #include "prot.h"
 #include "mailbox.h"
 
+/* cyrus.cache file item buffer */
+struct ibuf {
+    char *start, *end, *last;
+};
+
+/*
+ * Parsed form of a body-part
+ */
+struct body {
+    /* Content-* header information */
+    char *type;
+    char *subtype;
+    struct param *params;
+    char *id;
+    char *description;
+    char *encoding;
+    char *md5;
+    char *disposition;
+    struct param *disposition_params;
+    struct param *language;
+    char *location;
+
+    /* Location/size information */
+    long header_offset;
+    long header_size;
+    long header_lines;
+    long content_offset;
+    long content_size;
+    long content_lines;
+    long boundary_size;		/* Size of terminating boundary */
+    long boundary_lines;
+
+    int numparts;		/* For multipart types */
+    struct body *subpart;	/* For message/rfc822 and multipart types */
+
+    /*
+     * Other header information.
+     * Only meaningful for body-parts at top level or
+     * enclosed in message/rfc-822
+     */
+    char *date;
+    char *subject;
+    struct address *from;
+    struct address *sender;
+    struct address *reply_to;
+    struct address *to;
+    struct address *cc;
+    struct address *bcc;
+    char *in_reply_to;
+    char *message_id;
+    char *received_date;
+
+    /*
+     * Cached headers.  Only filled in at top-level
+     */
+    struct ibuf cacheheaders;
+
+    /*
+     * decoded body.  Filled in as needed.
+     */
+    char *decoded_body;
+
+    /* Message GUID. Only filled in at top level */
+    struct message_guid guid;
+};
+
+/* List of Content-type parameters */
+struct param {
+    struct param *next;
+    char *attribute;
+    char *value;
+};
 extern int message_copy_strict P((struct protstream *from, FILE *to,
 				  unsigned size, int allow_null));
 
+extern int message_parse(const char *fname, struct index_record *record);
+
 /* Flags for parsing message date/time - to be bitwise OR'd */
 #define PARSE_DATE	(1<<0)  /* Default (always parsed) */
 #define PARSE_TIME	(1<<1)
 #define PARSE_ZONE	(1<<2)
+#define PARSE_GMT	(1<<3) /* Output time in GMT rather than local timezone */
 #define PARSE_NOCREATE	(1<<15) /* Don't create one if its missing/invalid */
 
 extern time_t message_parse_date P((char *hdr, unsigned flags));
@@ -81,11 +156,16 @@ struct message_content {
 /* MUST keep this struct sync'd with sieve_bodypart in sieve_interface.h */
 struct bodypart {
     char section[128];
-    const char *content;
-    const char *encoding;
-    unsigned long size;
+    const char *decoded_body;
 };
 
+/* Calculate the number of entries in a vector */
+#define VECTOR_SIZE(vector) (sizeof(vector)/sizeof(vector[0]))
+
+extern void parse_cached_envelope P((char *env, char *tokens[], int tokens_size));
+
+extern int message_parse_mapped P((const char *msg_base, unsigned long msg_len,
+				   struct body *body));
 extern int message_parse_binary_file P((FILE *infile, struct body **body));
 extern int message_parse_file P((FILE *infile,
 				 const char **msg_base, unsigned long *msg_len,
@@ -93,9 +173,9 @@ extern int message_parse_file P((FILE *infile,
 extern void message_fetch_part P((struct message_content *msg,
 				  const char **content_types,
 				  struct bodypart ***parts));
-extern int message_create_record P((const char *cache_name,
-				    int cache_fd,
-				    struct index_record *message_index,
+extern int message_write_cache P((struct index_record *record, struct body *body));
+
+extern int message_create_record P((struct index_record *message_index,
 				    struct body *body));
 extern void message_free_body P((struct body *body));
 
