@@ -92,7 +92,7 @@ static struct protocol_t mupdate_protocol =
       { "* STARTTLS", CAPA_STARTTLS },
       { "* COMPRESS \"DEFLATE\"", CAPA_COMPRESS },
       { NULL, 0 } } },
-  { "S01 STARTTLS", "S01 OK", "S01 NO", 0 },
+  { "S01 STARTTLS", "S01 OK", "S01 NO", 1 },
   { "A01 AUTHENTICATE", USHRT_MAX, 1, "A01 OK", "A01 NO", "", "*", NULL, 0 },
   { "Z01 COMPRESS \"DEFLATE\"", NULL, "Z01 OK" },
   { "N01 NOOP", NULL, "N01 OK" },
@@ -664,6 +664,15 @@ void kick_mupdate(void)
     struct sockaddr_un srvaddr;
     int s, r;
     int len;
+
+    /* no server? drop out */
+    if (!config_mupdate_server)
+	return;
+
+    /* don't kick on standard backends */
+    if (config_mupdate_config == IMAP_ENUM_MUPDATE_CONFIG_STANDARD
+	&& config_getstring(IMAPOPT_PROXYSERVERS))
+	return;
     
     s = socket(AF_UNIX, SOCK_STREAM, 0);
     if (s == -1) {
