@@ -123,7 +123,6 @@ static int (*compar)(const char *s1, const char *s2);
 struct quotaentry *quota;
 int quota_num = 0, quota_alloc = 0;
 
-int firstquota = 0;
 int redofix = 0;
 
 int main(int argc,char **argv)
@@ -299,31 +298,25 @@ static int findroot(const char *name, int *thisquota)
 
     *thisquota = -1;
 
-    for (i = firstquota; i < quota_num; i++) {
+    for (i = 0; i < quota_num; i++) {
 	const char *root = quota[i].quota.root;
 
 	/* have we already passed the name, then there can
 	 * be no further matches */
 	if (compar(root, name) > 0)
-	    return 0;
+	    break;
 
 	/* is the mailbox within this root? */
 	if (mboxname_is_prefix(name, root)) {
 	    /* fantastic, but don't return yet, we may find
 	     * a more exact match */
-	    quota[i].refcount++;
 	    *thisquota = i;
 	}
-	else {
-	    /* not a match, so we can finish everything up to here */
-	    while (firstquota < i) {
-		int r = fixquota_finish(firstquota);
-		if (r) return r;
-		firstquota++;
-	    }
-	}
     }
 
+    if (*thisquota >= 0)
+	quota[*thisquota].refcount++;
+
     return 0;
 }
 
@@ -452,6 +445,7 @@ int fixquotas(char *domain, char **roots, int nroots)
     int i, r;
     char buf[MAX_MAILBOX_BUFFER], *tail;
     size_t domainlen = 0;
+    int firstquota = 0;
 
     buf[0] = '\0';
     tail = buf;
