@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: sync_log.h,v 1.6 2009/12/21 12:28:12 murch Exp $
+ * $Id: sync_log.h,v 1.7 2010/01/06 17:01:41 murch Exp $
  *
  * Original version written by David Carter <dpc22@cam.ac.uk>
  * Rewritten and integrated into Cyrus by Ken Murchison <ken@oceana.com>
@@ -51,9 +51,10 @@
 #define SYNC_LOG_RETRIES (64)
 
 void sync_log_init(void);
+void sync_log_suppress(void);
 void sync_log_done(void);
 
-void sync_log(char *fmt, ...);
+void sync_log(const char *fmt, ...);
 
 #define sync_log_user(user) \
     sync_log("USER %s\n", user)
@@ -67,12 +68,6 @@ void sync_log(char *fmt, ...);
 #define sync_log_mailbox_double(name1, name2) \
     sync_log("MAILBOX %s\nMAILBOX %s\n", name1, name2)
 
-#define sync_log_append(name) \
-    sync_log("APPEND %s\n", name)
-
-#define sync_log_acl(name) \
-    sync_log("ACL %s\n", name)
-
 #define sync_log_quota(name) \
     sync_log("QUOTA %s\n", name)
 
@@ -85,4 +80,32 @@ void sync_log(char *fmt, ...);
 #define sync_log_subscribe(user, name) \
     sync_log("SUB %s %s\n", user, name)
 
+char *sync_log_fname(const char *channel);
+void sync_log_suppress_channel(const char *channel);
+void sync_log_channel(const char *channel, const char *fmt, ...);
+
+#define sync_log_user_channel(channel, user) \
+    sync_log_channel(channel, "USER %s\n", user)
+
+#define sync_log_sieve_channel(channel, user) \
+    sync_log_channel(channel, "META %s\n", user)
+
+#define sync_log_mailbox_channel(channel, name) \
+    sync_log_channel(channel, "MAILBOX %s\n", name)
+
+#define sync_log_mailbox_double_channel(channel, name1, name2) \
+    sync_log_channel(channel, "MAILBOX %s\nMAILBOX %s\n", name1, name2)
+
+#define sync_log_quota_channel(channel, name) \
+    sync_log_channel(channel, "QUOTA %s\n", name)
+
+#define sync_log_annotation_channel(channel, name) \
+    sync_log_channel(channel, "ANNOTATION %s\n", name)
+
+#define sync_log_seen_channel(channel, user, name) \
+    sync_log_channel(channel, "SEEN %s %s\n", user, name)
+
+#define sync_log_subscribe_channel(channel, user, name) \
+    sync_log_channel(channel, "SUB %s %s\n", user, name)
+
 #endif /* INCLUDED_SYNC_LOG_H */
