@@ -43,7 +43,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: tls.c,v 1.66 2008/11/14 22:24:38 wescraig Exp $
+ * $Id: tls.c,v 1.67 2010/01/06 17:01:42 murch Exp $
  */
 
 /*
@@ -209,7 +209,6 @@ static void apps_ssl_info_callback(SSL * s, int where, int ret)
     }
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
 /* taken from OpenSSL apps/s_cb.c
    not thread safe! */
 static RSA *tmp_rsa_cb(SSL * s __attribute__((unused)),
@@ -223,51 +222,26 @@ static RSA *tmp_rsa_cb(SSL * s __attribute__((unused)),
     }
     return (rsa_tmp);
 }
-#endif
-
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-/* replacements for new 1.1 API accessors */
-/* XXX probably put these somewhere central */
-static int DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
-{
-    if (p == NULL || g == NULL) return 0;
-    dh->p = p;
-    dh->q = q; /* optional */
-    dh->g = g;
-    return 1;
-}
-#endif
 
 #if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
 /* Logic copied from OpenSSL apps/s_server.c: give the TLS context
  * DH params to work with DHE-* cipher suites. Hardcoded fallback
  * in case no DH params in tls_key_file or tls_cert_file.
- * Modified quite a bit for openssl 1.1.0 compatibility.
- * XXX we might be able to just replace this with DH_get_1024_160?
- * XXX the apps/s_server.c example doesn't use this anymore at all.
  */
 static DH *get_dh1024(void)
 {
     /* Second Oakley group 1024-bits MODP group from RFC2409 */
-    DH *dh;
-    BIGNUM *p = NULL, *g = NULL;
+    DH *dh=NULL;
 
-    dh = DH_new();
-    if (!dh) return NULL;
+    if ((dh=DH_new()) == NULL) return(NULL);
+    dh->p=get_rfc2409_prime_1024(NULL);
+    dh->g=NULL;
+    BN_dec2bn(&(dh->g), "2");
+    if ((dh->p == NULL) || (dh->g == NULL)) return(NULL);
 
-    p = get_rfc2409_prime_1024(NULL);
-    BN_dec2bn(&g, "2");
-
-    if (DH_set0_pqg(dh, p, NULL, g))
-	return dh;
-
-    if (g) BN_free(g);
-    if (p) BN_free(p);
-    DH_free(dh);
-
-    return NULL;
+    return(dh);
+	
 }
-
 static DH *load_dh_param(const char *keyfile, const char *certfile)
 {
     DH *ret=NULL;
@@ -321,9 +295,9 @@ static int verify_callback(int ok, X509_STORE_CTX * ctx)
 	    verify_error = X509_V_ERR_CERT_CHAIN_TOO_LONG;
 	}
     }
-    switch (err) {
+    switch (ctx->error) {
     case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
-	X509_NAME_oneline(X509_get_issuer_name(err_cert), buf, sizeof(buf));
+	X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, sizeof(buf));
 	syslog(LOG_NOTICE, "issuer= %s", buf);
 	break;
     case X509_V_ERR_CERT_NOT_YET_VALID:
@@ -470,8 +444,7 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
     unsigned char *data = NULL;
     time_t expire;
     int ret = -1;
-    const unsigned char *session_id = NULL;
-    unsigned int session_id_length = 0;
+    unsigned char *asn;
 
     assert(sess);
 
@@ -487,35 +460,31 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
     data = (unsigned char *) xmalloc(sizeof(time_t)+len*sizeof(unsigned char));
 
     /* transform the session into its ASN1 representation */
-    if (data) {
-	unsigned char *asn = data + sizeof(time_t);
-	len = i2d_SSL_SESSION(sess, &asn);
-	if (!len) syslog(LOG_ERR, "i2d_SSL_SESSION failed");
-    }
+    asn = data + sizeof(time_t);
+    len = i2d_SSL_SESSION(sess, &asn);
+    if (!len) syslog(LOG_ERR, "i2d_SSL_SESSION failed");
 
     /* set the expire time for the external cache, and prepend it to data */
     expire = SSL_SESSION_get_time(sess) + SSL_SESSION_get_timeout(sess);
     memcpy(data, &expire, sizeof(time_t));
 
-    if (data && len) {
+    if (len) {
 	/* store the session in our database */
-
-	session_id = SSL_SESSION_get_id(sess, &session_id_length);
 	do {
-	    ret = DB->store(sessdb, (const char *) session_id,
-			    session_id_length,
+	    ret = DB->store(sessdb, (const char *) sess->session_id,
+			    sess->session_id_length,
 			    (const char *) data, len + sizeof(time_t), NULL);
 	} while (ret == CYRUSDB_AGAIN);
     }
 
-    if (data) free(data);
+    free(data);
 
     /* log this transaction */
     if (var_imapd_tls_loglevel > 0) {
 	unsigned int i;
 	char idstr[SSL_MAX_SSL_SESSION_ID_LENGTH*2 + 1];
-	for (i = 0; i < session_id_length; i++) {
-	    sprintf(idstr+i*2, "%02X", session_id[i]);
+	for (i = 0; i < sess->session_id_length; i++) {
+	    sprintf(idstr+i*2, "%02X", sess->session_id[i]);
 	}
 	syslog(LOG_DEBUG, "new TLS session: id=%s, expire=%s, status=%s",
 	       idstr, ctime(&expire), ret ? "failed" : "ok");
@@ -527,7 +496,7 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
 /*
  * Function for removing a session from our database.
  */
-static void remove_session(const unsigned char *id, int idlen)
+static void remove_session(unsigned char *id, int idlen)
 {
     int ret;
 
@@ -562,14 +531,9 @@ static void remove_session(const unsigned char *id, int idlen)
 static void remove_session_cb(SSL_CTX *ctx __attribute__((unused)),
 			      SSL_SESSION *sess)
 {
-    const unsigned char *session_id = NULL;
-    unsigned int session_id_length = 0;
-
     assert(sess);
 
-    session_id = SSL_SESSION_get_id(sess, &session_id_length);
-
-    remove_session(session_id, session_id_length);
+    remove_session(sess->session_id, sess->session_id_length);
 }
 
 /*
@@ -578,13 +542,8 @@ static void remove_session_cb(SSL_CTX *ctx __attribute__((unused)),
  * called, also when session caching was disabled.  We lookup the
  * session in our database in case it was stored by another process.
  */
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-static SSL_SESSION *get_session_cb(SSL *ssl __attribute__((unused)),
-				   const unsigned char *id, int idlen, int *copy)
-#else
 static SSL_SESSION *get_session_cb(SSL *ssl __attribute__((unused)),
 				   unsigned char *id, int idlen, int *copy)
-#endif
 {
     int ret;
     const char *data = NULL;
@@ -672,6 +631,7 @@ int     tls_init_serverengine(const char *ident,
     const char   *s_cert_file;
     const char   *s_key_file;
     int    requirecert;
+    int    server_cipher_order;
     int    timeout;
 
     if (tls_serverengine)
@@ -687,13 +647,6 @@ int     tls_init_serverengine(const char *ident,
 	return -1;
     }
 
-#if 0
-    if (tlsonly) {
-	s_ctx = SSL_CTX_new(TLSv1_server_method());
-    } else {
-	s_ctx = SSL_CTX_new(SSLv23_server_method());
-    }
-#endif
     /* even if we want TLS only, we use SSLv23 server method so we can
        deal with a client sending an SSLv2 greeting message */
 
@@ -703,12 +656,106 @@ int     tls_init_serverengine(const char *ident,
     };
 
     off |= SSL_OP_ALL;		/* Work around all known bugs */
-	off |= SSL_OP_NO_SSLv2;
-	off |= SSL_OP_NO_SSLv3;
+
+    const char *tls_versions = config_getstring(IMAPOPT_TLS_VERSIONS);
+
+    off |= SSL_OP_NO_SSLv2;
+    off |= SSL_OP_NO_SSLv3;
+
+    if (strstr(tls_versions, "tls1_2") == NULL) {
+#if (OPENSSL_VERSION_NUMBER >= 0x1000105fL)
+	off |= SSL_OP_NO_TLSv1_2;
+#else
+	syslog(LOG_ERR, "ERROR: TLSv1.2 configured, OpenSSL < 1.0.1e insufficient");
+#endif
+    }
+
+    if (strstr(tls_versions, "tls1_1") == NULL) {
+#if (OPENSSL_VERSION_NUMBER >= 0x1000000fL)
+	off |= SSL_OP_NO_TLSv1_1;
+#else
+	syslog(LOG_ERR, "ERROR: TLSv1.1 configured, OpenSSL < 1.0.0 insufficient");
+#endif
+    }
+    if (strstr(tls_versions, "tls1_0") == NULL) {
+	off |= SSL_OP_NO_TLSv1;
+    }
+
+    server_cipher_order = config_getswitch(IMAPOPT_TLS_PREFER_SERVER_CIPHERS);
+    if (server_cipher_order)
+        off |= SSL_OP_CIPHER_SERVER_PREFERENCE;
 
     SSL_CTX_set_options(s_ctx, off);
     SSL_CTX_set_info_callback(s_ctx, (void (*)()) apps_ssl_info_callback);
 
+    cipher_list = config_getstring(IMAPOPT_TLS_CIPHER_LIST);
+    if (!SSL_CTX_set_cipher_list(s_ctx, cipher_list)) {
+	syslog(LOG_ERR,"TLS server engine: cannot load cipher list '%s'",
+	       cipher_list);
+	return (-1);
+    }
+
+    CAfile = config_getstring(IMAPOPT_TLS_CA_FILE);
+    CApath = config_getstring(IMAPOPT_TLS_CA_PATH);
+
+    if (CAfile || CApath) {
+	if ((!SSL_CTX_load_verify_locations(s_ctx, CAfile, CApath)) ||
+	    (!SSL_CTX_set_default_verify_paths(s_ctx))) {
+	    /* just a warning since this is only necessary for client auth */
+	    syslog(LOG_NOTICE,"TLS server engine: cannot load CA data. Disabling client certs.");
+	    askcert = 0;
+	}
+    } else {
+	askcert = 0;
+    }
+
+    s_cert_file = config_getstring(IMAPOPT_TLS_CERT_FILE);
+    s_key_file = config_getstring(IMAPOPT_TLS_KEY_FILE);
+
+    if (!set_cert_stuff(s_ctx, s_cert_file, s_key_file)) {
+	syslog(LOG_ERR,"TLS server engine: cannot load cert/key data");
+	return (-1);
+    }
+    SSL_CTX_set_tmp_rsa_callback(s_ctx, tmp_rsa_cb);
+
+#if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
+    /* Load DH params for DHE-* key exchanges */
+    SSL_CTX_set_tmp_dh(s_ctx, load_dh_param(s_key_file, s_cert_file));
+#endif
+
+#if (OPENSSL_VERSION_NUMBER >= 0x1000103fL)
+    const char *ec = config_getstring(IMAPOPT_TLS_ECCURVE);
+    int openssl_nid = OBJ_sn2nid(ec);
+    if (openssl_nid != 0) {
+	EC_KEY *ecdh;
+	ecdh = EC_KEY_new_by_curve_name(openssl_nid);
+	if (ecdh != NULL) {
+	    SSL_CTX_set_tmp_ecdh(s_ctx, ecdh);
+	    EC_KEY_free(ecdh);
+	}
+    }
+#endif
+
+    verify_depth = verifydepth;
+    if (askcert != 0)
+	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE;
+
+    requirecert = config_getswitch(IMAPOPT_TLS_REQUIRE_CERT);
+    if (requirecert)
+	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT
+	    | SSL_VERIFY_CLIENT_ONCE;
+    SSL_CTX_set_verify(s_ctx, verify_flags, verify_callback);
+
+    if (askcert || requirecert) {
+      if (CAfile == NULL) {
+	  syslog(LOG_ERR, 
+		 "TLS server engine: No CA file specified. "
+		 "Client side certs will not work.");
+      } else {
+	  SSL_CTX_set_client_CA_list(s_ctx, SSL_load_client_CA_file(CAfile));
+      }
+    }
+
     /* Don't use an internal session cache */
     SSL_CTX_sess_set_cache_size(s_ctx, 1);  /* 0 is unlimited, so use 1 */
     SSL_CTX_set_session_cache_mode(s_ctx, SSL_SESS_CACHE_SERVER |
@@ -722,7 +769,8 @@ int     tls_init_serverengine(const char *ident,
 
     /* A timeout of zero disables session caching */
     if (timeout) {
-	char dbdir[1024];
+	const char *fname = NULL;
+	char *tofree = NULL;
 	int r;
 
 	/* Set the context for session reuse -- use the service ident */
@@ -736,71 +784,23 @@ int     tls_init_serverengine(const char *ident,
 	SSL_CTX_sess_set_remove_cb(s_ctx, remove_session_cb);
 	SSL_CTX_sess_set_get_cb(s_ctx, get_session_cb);
 
-	/* create the name of the db file */
-	strlcpy(dbdir, config_dir, sizeof(dbdir));
-	strlcat(dbdir, FNAME_TLSSESSIONS, sizeof(dbdir));
+	fname = config_getstring(IMAPOPT_TLSCACHE_DB_PATH);
 
-	r = (DB->open)(dbdir, CYRUSDB_CREATE, &sessdb);
+	/* create the name of the db file */
+	if (!fname) {
+	    tofree = strconcat(config_dir, FNAME_TLSSESSIONS, (char *)NULL);
+	    fname = tofree;
+	}
+
+	r = (DB->open)(fname, CYRUSDB_CREATE, &sessdb);
 	if (r != 0) {
 	    syslog(LOG_ERR, "DBERROR: opening %s: %s",
-		   dbdir, cyrusdb_strerror(ret));
+		   fname, cyrusdb_strerror(ret));
 	}
 	else
 	    sess_dbopen = 1;
-    }
 
-    cipher_list = config_getstring(IMAPOPT_TLS_CIPHER_LIST);
-    if (!SSL_CTX_set_cipher_list(s_ctx, cipher_list)) {
-	syslog(LOG_ERR,"TLS server engine: cannot load cipher list '%s'",
-	       cipher_list);
-	return (-1);
-    }
-
-    CAfile = config_getstring(IMAPOPT_TLS_CA_FILE);
-    CApath = config_getstring(IMAPOPT_TLS_CA_PATH);
-
-    if ((!SSL_CTX_load_verify_locations(s_ctx, CAfile, CApath)) ||
-	(!SSL_CTX_set_default_verify_paths(s_ctx))) {
-	/* just a warning since this is only necessary for client auth */
-	syslog(LOG_NOTICE,"TLS server engine: cannot load CA data");	
-    }
-
-    s_cert_file = config_getstring(IMAPOPT_TLS_CERT_FILE);
-    s_key_file = config_getstring(IMAPOPT_TLS_KEY_FILE);
-
-    if (!set_cert_stuff(s_ctx, s_cert_file, s_key_file)) {
-	syslog(LOG_ERR,"TLS server engine: cannot load cert/key data");
-	return (-1);
-    }
-
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-    SSL_CTX_set_tmp_rsa_callback(s_ctx, tmp_rsa_cb);
-#endif
-
-#if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
-    /* Load DH params for DHE-* key exchanges */
-    SSL_CTX_set_tmp_dh(s_ctx, load_dh_param(s_key_file, s_cert_file));
-    /* FIXME: Load ECDH params for ECDHE suites when 0.9.9 is released */
-#endif
-
-    verify_depth = verifydepth;
-    if (askcert!=0)
-	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE;
-
-    requirecert = config_getswitch(IMAPOPT_TLS_REQUIRE_CERT);
-    if (requirecert)
-	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT
-	    | SSL_VERIFY_CLIENT_ONCE;
-    SSL_CTX_set_verify(s_ctx, verify_flags, verify_callback);
-
-    if (askcert || requirecert) {
-      if (CAfile == NULL) {
-	  syslog(LOG_ERR, 
-		 "TLS server engine: No CA file specified. "
-		 "Client side certs may not work");
-      } else {
-	  SSL_CTX_set_client_CA_list(s_ctx, SSL_load_client_CA_file(CAfile));
-      }
+	free(tofree);
     }
 
     tls_serverengine = 1;
@@ -817,15 +817,15 @@ static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
 	return (ret);
 
     if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
-	printf("read from %08X [%08lX] (%d bytes => %ld (0x%X))",
-	       (unsigned int) bio, (long unsigned int) argp,
-	       argi, ret, (unsigned int) ret);
+	printf("read from %08lX [%08lX] (%d bytes => %ld (0x%lX))",
+	       (unsigned long)bio, (unsigned long)argp,
+	       argi, ret, ret);
 	tls_dump(argp, (int) ret);
 	return (ret);
     } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
-	printf("write to %08X [%08lX] (%d bytes => %ld (0x%X))",
-	       (unsigned int) bio, (long unsigned int)argp,
-	       argi, ret, (unsigned int) ret);
+	printf("write to %08lX [%08lX] (%d bytes => %ld (0x%lX))",
+	       (unsigned long) bio, (unsigned long)argp,
+	       argi, ret, ret);
 	tls_dump(argp, (int) ret);
     }
     return (ret);
@@ -850,7 +850,7 @@ int tls_start_servertls(int readfd, int writefd, int timeout,
     int     sts;
     int     j;
     unsigned int n;
-    SSL_CIPHER *cipher;
+    const SSL_CIPHER *cipher;
     X509   *peer;
     const char *tls_protocol = NULL;
     const char *tls_cipher_name = NULL;
@@ -910,7 +910,7 @@ int tls_start_servertls(int readfd, int writefd, int timeout,
 	tv.tv_sec = timeout;
 	tv.tv_usec = 0;
 
-	sts = select(1, &rfds, NULL, NULL, &tv);
+	sts = select(readfd+1, &rfds, NULL, NULL, &tv);
 	if (sts <= 0) {
 	    if (sts == 0) {
 		syslog(LOG_DEBUG, "SSL_accept() timed out -> fail");
@@ -1169,18 +1169,23 @@ static int prune_cb(void *rock, const char *id, int idlen,
 /* must be called after cyrus_init */
 int tls_prune_sessions(void)
 {
-    char dbdir[1024];
+    const char *fname = NULL;
+    char *tofree = NULL;
     int ret;
     struct prunerock prock;
 
-   /* create the name of the db file */
-    strlcpy(dbdir, config_dir, sizeof(dbdir));
-    strlcat(dbdir, FNAME_TLSSESSIONS, sizeof(dbdir));
+    fname = config_getstring(IMAPOPT_TLSCACHE_DB_PATH);
 
-    ret = (DB->open)(dbdir, 0, &sessdb);
+   /* create the name of the db file */
+    if (!fname) {
+	tofree = strconcat(config_dir, FNAME_TLSSESSIONS, (char *)NULL);
+	fname = tofree;
+    }
+
+    ret = (DB->open)(fname, 0, &sessdb);
     if (ret != CYRUSDB_OK) {
 	syslog(LOG_ERR, "DBERROR: opening %s: %s",
-	       dbdir, cyrusdb_strerror(ret));
+	       fname, cyrusdb_strerror(ret));
 	return 1;
     }
     else {
@@ -1196,6 +1201,8 @@ int tls_prune_sessions(void)
 	       prock.deletions, prock.count);
     }
 
+    free(tofree);
+
     return 0;
 }
 
@@ -1238,14 +1245,14 @@ int tls_init_clientengine(int verifydepth,
 	return -1;
     }
     
-    c_ctx = SSL_CTX_new(TLSv1_client_method());
+    c_ctx = SSL_CTX_new(SSLv23_client_method());
     if (c_ctx == NULL) {
 	return (-1);
     };
     
     off |= SSL_OP_ALL;		/* Work around all known bugs */
-	off |= SSL_OP_NO_SSLv2;
-	off |= SSL_OP_NO_SSLv3;
+    off |= SSL_OP_NO_SSLv2;
+    off |= SSL_OP_NO_SSLv3;
 
     SSL_CTX_set_options(c_ctx, off);
     SSL_CTX_set_info_callback(c_ctx, (void (*)()) apps_ssl_info_callback);
@@ -1256,7 +1263,7 @@ int tls_init_clientengine(int verifydepth,
     if ((!SSL_CTX_load_verify_locations(c_ctx, CAfile, CApath)) ||
 	(!SSL_CTX_set_default_verify_paths(c_ctx))) {
 	/* just a warning since this is only necessary for client auth */
-	syslog(LOG_NOTICE,"TLS client engine: cannot load CA data");	
+	syslog(LOG_NOTICE,"TLS client engine: cannot load CA data");
     }
 
     if (strlen(var_tls_cert_file) == 0)
@@ -1274,10 +1281,7 @@ int tls_init_clientengine(int verifydepth,
 	    return (-1);
 	}
     }
-
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
     SSL_CTX_set_tmp_rsa_callback(c_ctx, tmp_rsa_cb);
-#endif
     
     verify_depth = verifydepth;
     SSL_CTX_set_verify(c_ctx, verify_flags, verify_callback);
@@ -1291,7 +1295,7 @@ int tls_start_clienttls(int readfd, int writefd,
 			SSL_SESSION **sess)
 {
     int     sts;
-    SSL_CIPHER *cipher;
+    const SSL_CIPHER *cipher;
     X509   *peer;
     const char *tls_protocol = NULL;
     const char *tls_cipher_name = NULL;
