@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: user.c,v 1.27 2009/03/31 16:54:45 murch Exp $
+ * $Id: user.c,v 1.28 2010/01/06 17:01:42 murch Exp $
  */
 
 #include <config.h>
@@ -81,6 +81,9 @@
 #include "seen.h"
 #include "quota.h"
 #include "xmalloc.h"
+#include "sync_log.h"
+
+#define FNAME_SUBSSUFFIX ".sub"
 
 #if 0
 static int user_deleteacl(char *name, int matchlen, int maycreate, void* rock)
@@ -122,17 +125,11 @@ static int user_deleteacl(char *name, int matchlen, int maycreate, void* rock)
 }
 #endif
 
-static int user_deletesieve(char *user) 
+const char *user_sieve_path(const char *user)
 {
+    static char sieve_path[2048];
     char hash, *domain;
-    char sieve_path[2048];
-    char filename[2048];
-    DIR *mbdir;
-    struct dirent *next = NULL;
-    
-    /* oh well */
-    if(config_getswitch(IMAPOPT_SIEVEUSEHOMEDIR)) return 0;
-    
+
     if (config_virtdomains && (domain = strchr(user, '@'))) {
 	char d = (char) dir_hash_c(domain+1, config_fulldirhash);
 	*domain = '\0';  /* split user@domain */
@@ -149,12 +146,27 @@ static int user_deletesieve(char *user)
 		 config_getstring(IMAPOPT_SIEVEDIR), hash, user);
     }
 
-    mbdir=opendir(sieve_path);
+    return sieve_path;
+}
 
-    if(mbdir) {
+static int user_deletesieve(const char *user) 
+{
+    const char *sieve_path;
+    char filename[2048];
+    DIR *mbdir;
+    struct dirent *next = NULL;
+    
+    /* oh well */
+    if(config_getswitch(IMAPOPT_SIEVEUSEHOMEDIR)) return 0;
+
+    sieve_path = user_sieve_path(user);
+
+    mbdir = opendir(sieve_path);
+
+    if (mbdir) {
 	while((next = readdir(mbdir)) != NULL) {
-	    if(!strcmp(next->d_name, ".")
-	       || !strcmp(next->d_name, "..")) continue;
+	    if (!strcmp(next->d_name, ".")
+	        || !strcmp(next->d_name, "..")) continue;
 
 	    snprintf(filename, sizeof(filename), "%s/%s",
 		     sieve_path, next->d_name);
@@ -185,7 +197,7 @@ int user_deletedata(char *user, char *userid __attribute__((unused)),
     }
 
     /* delete subscriptions */
-    fname = mboxlist_hash_usersubs(user);
+    fname = user_hash_subs(user);
     (void) unlink(fname);
     free(fname);
 
@@ -205,6 +217,8 @@ int user_deletedata(char *user, char *userid __attribute__((unused)),
 
     /* delete sieve scripts */
     user_deletesieve(user);
+
+    sync_log_user(user);
     
     return 0;
 }
@@ -243,9 +257,7 @@ static int user_renamesub(char *name, int matchlen __attribute__((unused)),
 	return 0;
     }
 
-    mboxlist_changesub(name, rrock->newuser, NULL, 1, 1);
-
-    return 0;
+    return mboxlist_changesub(name, rrock->newuser, NULL, 1, 1);
 }
 
 static int user_renamesieve(char *olduser, char *newuser)
@@ -377,12 +389,14 @@ int user_renameacl(char *name, char *olduser, char *newuser)
     int r = 0;
     char *acl;
     char *rights, *nextid;
-    char *origacl, *aclalloc;
+    struct mboxlist_entry mbentry;
+    char *aclalloc;
 
-    r = mboxlist_lookup(name, &origacl, NULL);
+    r = mboxlist_lookup(name, &mbentry, NULL);
+    if (r) return r;
 
     /* setacl re-calls mboxlist_lookup and will stomp on us */
-    aclalloc = acl = xstrdup(origacl);
+    aclalloc = acl = xstrdup(mbentry.acl);
 
     while (!r && acl) {
 	rights = strchr(acl, '\t');
@@ -412,44 +426,39 @@ int user_renameacl(char *name, char *olduser, char *newuser)
 int user_copyquotaroot(char *oldname, char *newname)
 {
     int r = 0;
-    struct quota quota;
+    struct quota q;
 
-    quota.root = oldname;
-
-    r = quota_read(&quota, NULL, 0);
-    if (!r) mboxlist_setquota(newname, quota.limit, 0);
+    q.root = oldname;
+    r = quota_read(&q, NULL, 0);
+    if (!r) mboxlist_setquota(newname, q.limit, 0);
 
     return r;
 }
 
-struct find_rock {
-    char *inboxname;
-    struct txn **tid;
-};
-
 static int find_p(void *rockp,
 		  const char *key, int keylen,
 		  const char *data __attribute__((unused)),
 		  int datalen __attribute__((unused)))
 {
-    char *inboxname = ((struct find_rock *) rockp)->inboxname;
+    char *inboxname = (char *)rockp;
+    int inboxlen = strlen(inboxname);
 
-    return (!strncmp(key, inboxname, strlen(inboxname)) &&
-	    (keylen == (int) strlen(inboxname) || key[strlen(inboxname)] == '.'));
+    return (!strncmp(key, inboxname, inboxlen) &&
+	    (keylen == inboxlen || key[inboxlen] == '.'));
 }
 
-static int find_cb(void *rockp,
-		  const char *key, int keylen,
-		  const char *data __attribute__((unused)),
+static int find_cb(void *rockp __attribute__((unused)),
+		   const char *key, int keylen,
+		   const char *data __attribute__((unused)),
 		   int datalen __attribute__((unused)))
 {
-    struct quota quota_root;
-    struct txn **tid = ((struct find_rock *) rockp)->tid;
+    char *root;
     int r;
 
-    quota_root.root = (char *) xstrndup(key, keylen);
-    r = quota_delete(&quota_root, tid);
-    free(quota_root.root); 
+    root = xstrndup(key, keylen);
+    r = quota_deleteroot(root);
+    free(root); 
+
     return r;
 }
 
@@ -457,7 +466,6 @@ int user_deletequotaroots(const char *user)
 {
     struct namespace namespace;
     char buf[MAX_MAILBOX_BUFFER], *inboxname = buf;
-    struct txn *tid = NULL;
     int r;
 
     /* set namespace */
@@ -470,14 +478,36 @@ int user_deletequotaroots(const char *user)
     }
 
     if (!r) {
-	struct find_rock frock = { NULL, NULL };
-	frock.inboxname = inboxname;
-	frock.tid = &tid;
 	r = config_quota_db->foreach(qdb, inboxname, strlen(inboxname),
-				     &find_p, &find_cb, &frock, &tid);
+				     &find_p, &find_cb, inboxname, NULL);
     }
 
-    if (!r) quota_commit(&tid);
-
     return r;
 }
+
+/* hash the userid to a file containing the subscriptions for that user */
+char *user_hash_subs(const char *userid)
+{
+    char *fname = xmalloc(strlen(config_dir) + sizeof(FNAME_DOMAINDIR) +
+			  sizeof(FNAME_USERDIR) + strlen(userid) +
+			  sizeof(FNAME_SUBSSUFFIX) + 10);
+    char c, *domain;
+
+    if (config_virtdomains && (domain = strchr(userid, '@'))) {
+	char d = (char) dir_hash_c(domain+1, config_fulldirhash);
+	*domain = '\0';  /* split user@domain */
+	c = (char) dir_hash_c(userid, config_fulldirhash);
+	sprintf(fname, "%s%s%c/%s%s%c/%s%s", config_dir, FNAME_DOMAINDIR, d,
+		domain+1, FNAME_USERDIR, c, userid, FNAME_SUBSSUFFIX);
+	*domain = '@';  /* replace '@' */
+    }
+    else {
+	c = (char) dir_hash_c(userid, config_fulldirhash);
+	sprintf(fname, "%s%s%c/%s%s", config_dir, FNAME_USERDIR, c, userid,
+		FNAME_SUBSSUFFIX);
+    }
+
+    return fname;
+}
+
+
