@@ -39,38 +39,11 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: version.h,v 1.163 2009/12/21 11:42:01 murch Exp $
+ * $Id: version.h,v 1.164 2010/01/06 17:01:42 murch Exp $
  */
 
-#define _CYRUS_VERSION "v2.3.16"
-
-/* EXTRA_IDENT is a hack to add some version information for which compile
- * was used to build this version (at CMU, but we don't care what you do with
- * it).
- */
-
-#ifdef EXTRA_IDENT
-#define CYRUS_VERSION _CYRUS_VERSION "-" EXTRA_IDENT
-#else
-#define CYRUS_VERSION _CYRUS_VERSION
-#endif
-
-enum {
-    CAPA_PREAUTH = 0x1,
-    CAPA_POSTAUTH = 0x2
-};
-
-/* CAPABILITIES are defined here, not including TLS/SASL ones,
-   and those that are configurable */
-#define CAPA_PREAUTH_STRING "IMAP4 IMAP4rev1 LITERAL+ ID"
-
-#define CAPA_POSTAUTH_STRING " ACL RIGHTS=kxte QUOTA " \
-	"MAILBOX-REFERRALS NAMESPACE UIDPLUS " \
-	"NO_ATOMIC_RENAME UNSELECT " \
-	"CHILDREN MULTIAPPEND BINARY " \
-	"SORT SORT=MODSEQ THREAD=ORDEREDSUBJECT THREAD=REFERENCES " \
-	"ANNOTATEMORE CATENATE CONDSTORE SCAN"
-
+#ifndef _CYRUS_VERSION_H
+#define _CYRUS_VERSION_H
 
 /* Values for ID processing */
 enum {
@@ -81,3 +54,7 @@ enum {
     MAXIDPAIRS = 30,
     MAXIDLOGLEN = (MAXIDPAIRS * (MAXIDFIELDLEN + MAXIDVALUELEN + 6))
 };
+
+const char *cyrus_version();
+
+#endif /* _CYRUS_VERSION_H */
