@@ -506,7 +506,7 @@ static int tls_init_clientengine(int verifydepth, char *var_tls_cert_file, char
 	return IMTEST_FAIL;
     }
     
-    tls_ctx = SSL_CTX_new(TLSv1_client_method());
+    tls_ctx = SSL_CTX_new(SSLv23_client_method());
     if (tls_ctx == NULL) {
 	return IMTEST_FAIL;
     };
@@ -652,7 +652,7 @@ static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
 int tls_start_clienttls(unsigned *layer, char **authid)
 {
     int     sts;
-    SSL_CIPHER *cipher;
+    const SSL_CIPHER *cipher;
     X509   *peer;
     
     if (verbose==1)
@@ -1204,6 +1204,20 @@ static void sigint_handler(int sig __attribute__((unused)))
     gotsigint = 1;
 }
 
+static int haveinput(struct protstream *s)
+{
+    /* Is something currently pending in our protstream's buffer? */
+#ifdef HAVE_SSL
+    if (s->cnt == 0 && s->tls_conn != NULL) {
+	/* Maybe there's data pending in the SSL buffer? */
+	int n = SSL_pending(s->tls_conn);
+	if (verbose) printf("SSL_pending=%d\n", n);
+	return n;
+    }
+#endif
+    return s->cnt;
+}
+
 /* This needs to support 3 modes:
  *
  * 1. Terminal Interface Only
@@ -1368,7 +1382,7 @@ static void interactive(struct protocol_t *protocol, char *filename)
 		    buf[count] = '\0';
 		    printf("%s", buf); 
 		}
-	    } while (pin->cnt > 0);
+	    } while (haveinput(pin) > 0);
 	} else if ((FD_ISSET(fd, &rset)) && (FD_ISSET(sock, &wset))
 		   && (donewritingfile == 0)) {
 	    /* This does input for both socket and file modes */
@@ -2736,7 +2750,13 @@ int main(int argc, char **argv)
 	} else {
 	    const char *s = sasl_errstring(result, NULL, NULL);
 	    
-	    printf("Authentication failed. %s\n", s);
+	    fprintf(stderr, "Authentication failed. %s\n", s);
+
+	    if (output_socket) {
+		/* send LOGOUT and exit */
+		logout(&protocol->logout_cmd, 0);
+		exit(1);
+	    }
 	}
 	
 	result = sasl_getprop(conn, SASL_SSF, &ssfp);
