@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: charset.c,v 1.54.2.1 2010/02/12 03:38:21 brong Exp $
  */
 
 #include <config.h>
@@ -53,16 +51,6 @@
 #include "chartable.h"
 #include "util.h"
 
-/* unicode canon translations */
-extern const int chartables_translation_multichar[];
-extern const unsigned char chartables_translation_block16[256];
-extern const unsigned char chartables_translation_block8[][256];
-extern const int chartables_translation[][256];
-
-/* named character sets */
-extern const struct charset chartables_charset_table[];
-extern const int chartables_num_charsets;
-
 struct qp_state {
     int isheader;
     int bytesleft;
@@ -94,7 +82,7 @@ struct comp_pat_s {
 };
 
 struct search_state {
-    size_t *starts;
+    ssize_t *starts;
     int max_start;
     int havematch;
     unsigned char *substr;
@@ -102,12 +90,6 @@ struct search_state {
     size_t offset;
 };
 
-struct buffer_state {
-    unsigned char *base;
-    size_t offset;
-    size_t alloced;
-};
-
 struct convert_rock;
 
 typedef void convertproc_t(struct convert_rock *rock, int c);
@@ -174,7 +156,7 @@ static inline void convert_putc(struct convert_rock *rock, int c)
     rock->f(rock, c);
 }
 
-void convert_cat(struct convert_rock *rock, const char *s)
+static void convert_cat(struct convert_rock *rock, const char *s)
 {
     while (*s) {
 	convert_putc(rock, (unsigned char)*s);
@@ -182,7 +164,7 @@ void convert_cat(struct convert_rock *rock, const char *s)
     }
 }
 
-void convert_catn(struct convert_rock *rock, const char *s, size_t len)
+static void convert_catn(struct convert_rock *rock, const char *s, size_t len)
 {
     while (len-- > 0) {
 	convert_putc(rock, (unsigned char)*s);
@@ -192,7 +174,7 @@ void convert_catn(struct convert_rock *rock, const char *s, size_t len)
 
 /* convertproc_t conversion functions */
 
-void qp2byte(struct convert_rock *rock, int c) 
+static void qp2byte(struct convert_rock *rock, int c)
 {
     struct qp_state *s = (struct qp_state *)rock->state;
     int val;
@@ -232,7 +214,7 @@ void qp2byte(struct convert_rock *rock, int c)
     convert_putc(rock->next, c);
 }
 
-void b64_2byte(struct convert_rock *rock, int c) 
+static void b64_2byte(struct convert_rock *rock, int c)
 {
     struct b64_state *s = (struct b64_state *)rock->state;
     char b = CHAR64(c);
@@ -262,18 +244,18 @@ void b64_2byte(struct convert_rock *rock, int c)
     }
 }
 
-void stripnl2uni(struct convert_rock *rock, int c)
+static void stripnl2uni(struct convert_rock *rock, int c)
 {
     if (c != '\r' && c != '\n')
 	convert_putc(rock->next, c);
 }
 
-void table2uni(struct convert_rock *rock, int c)
+static void table2uni(struct convert_rock *rock, int c)
 {
     struct table_state *s = (struct table_state *)rock->state;
     struct charmap *map = (struct charmap *)&s->curtable[0][c & 0xff];
 
-    /* propogate errors */
+    /* propagate errors */
     if (c == 0xfffd) {
 	convert_putc(rock->next, c);
 	return;
@@ -285,11 +267,11 @@ void table2uni(struct convert_rock *rock, int c)
     s->curtable = s->initialtable + map->next;
 }
 
-void utf8_2uni(struct convert_rock *rock, int c)
+static void utf8_2uni(struct convert_rock *rock, int c)
 {
     struct table_state *s = (struct table_state *)rock->state;
 
-    /* propogate errors */
+    /* propagate errors */
     if (c == 0xfffd) {
 	convert_putc(rock->next, c);
 	return;
@@ -328,11 +310,11 @@ void utf8_2uni(struct convert_rock *rock, int c)
     }
 }
 
-void utf7_2uni (struct convert_rock *rock, int c)
+static void utf7_2uni (struct convert_rock *rock, int c)
 {
     struct table_state *s = (struct table_state *)rock->state;
 
-    /* propogate errors */
+    /* propagate errors */
     if (c == 0xfffd) {
 	convert_putc(rock->next, c);
 	return;
@@ -396,7 +378,7 @@ void utf7_2uni (struct convert_rock *rock, int c)
     }
 }
 
-void uni2searchform(struct convert_rock *rock, int c)
+static void uni2searchform(struct convert_rock *rock, int c)
 {
     struct canon_state *s = (struct canon_state *)rock->state;
     int i;
@@ -466,7 +448,7 @@ void uni2searchform(struct convert_rock *rock, int c)
     }
 }
 
-void uni2utf8(struct convert_rock *rock, int c)
+static void uni2utf8(struct convert_rock *rock, int c)
 {
     if (c > 0xffff) {
 	convert_putc(rock->next, 0xF0 + ((c >> 18) & 0x07));
@@ -488,7 +470,7 @@ void uni2utf8(struct convert_rock *rock, int c)
     }
 }
 
-void byte2search(struct convert_rock *rock, int c)
+static void byte2search(struct convert_rock *rock, int c)
 {
     struct search_state *s = (struct search_state *)rock->state;
     int i, cur;
@@ -501,7 +483,7 @@ void byte2search(struct convert_rock *rock, int c)
     /* check our "in_progress" matches to see if they're still valid */
     for (i = 0, cur = 0; i < s->max_start; i++) {
 	/* no more active offsets */
-	if (s->starts[i] == -1) 
+	if (s->starts[i] == -1)
 	    break;
 
 	/* if we've passed one that's not ongoing, copy back */
@@ -525,7 +507,7 @@ void byte2search(struct convert_rock *rock, int c)
 	/* have to treat this one specially! */
 	if (s->patlen == 1)
 	    s->havematch = 1;
-	else 
+	else
 	    s->starts[cur++] = s->offset;
     }
     /* empty out any others that aren't being kept */
@@ -535,22 +517,16 @@ void byte2search(struct convert_rock *rock, int c)
     s->offset++;
 }
 
-void byte2buffer(struct convert_rock *rock, int c)
+static void byte2buffer(struct convert_rock *rock, int c)
 {
-    struct buffer_state *buf = (struct buffer_state *)rock->state;
+    struct buf *buf = (struct buf *)rock->state;
 
-    /* make sure we have the space */
-    if (buf->offset >= buf->alloced) {
-	buf->alloced += GROWSIZE;
-	buf->base = realloc(buf->base, buf->alloced);
-    }
-
-    buf->base[buf->offset++] = c & 0xff;
+    buf_putc(buf, c & 0xff);
 }
 
 /* convert_rock manipulation routines */
 
-void table_switch(struct convert_rock *rock, int charset_num)
+static void table_switch(struct convert_rock *rock, int charset_num)
 {
     struct table_state *state = (struct table_state *)rock->state;
 
@@ -585,25 +561,18 @@ void table_switch(struct convert_rock *rock, int charset_num)
 /* Extract a cstring from a buffer.  NOTE: caller must free the memory
  * themselves once this is called.  Resets the state.  If you don't
  * call this function then buffer_free will clean up */
-char *buffer_cstring(struct convert_rock *rock)
+static unsigned char *buffer_cstring(struct convert_rock *rock)
 {
-    struct buffer_state *buf = (struct buffer_state *)rock->state;
-    char *res;
-
-    /* finish the string */
-    if (buf->offset >= buf->alloced) {
-	buf->alloced++;
-	buf->base = realloc(buf->base, buf->alloced);
-    }
-    buf->base[buf->offset] = '\0';
+    struct buf *buf = (struct buf *)rock->state;
+    unsigned char *res;
 
     /* copy the pointer out */
-    res = buf->base;
+    res = (unsigned char *)buf_cstring(buf);
 
     /* clean up the buffer so it frees correctly later */
-    buf->base = 0;
-    buf->alloced = 0;
-    buf->offset = 0;
+    buf->s = 0;
+    buf->len = 0;
+    buf->alloc = 0;
 
     return res;
 }
@@ -616,7 +585,7 @@ static inline int search_havematch(struct convert_rock *rock)
 
 /* conversion cleanup routines */
 
-void basic_free(struct convert_rock *rock) 
+static void basic_free(struct convert_rock *rock)
 {
     if (rock) {
 	if (rock->state) free(rock->state);
@@ -624,7 +593,7 @@ void basic_free(struct convert_rock *rock)
     }
 }
 
-void search_free(struct convert_rock *rock)
+static void search_free(struct convert_rock *rock)
 {
     if (rock && rock->state) {
 	struct search_state *s = (struct search_state *)rock->state;
@@ -633,15 +602,16 @@ void search_free(struct convert_rock *rock)
     basic_free(rock);
 }
 
-void buffer_free(struct convert_rock *rock) {
+static void buffer_free(struct convert_rock *rock)
+{
     if (rock && rock->state) {
-	struct buffer_state *buf = (struct buffer_state *)rock->state;
-	if (buf->base) free(buf->base);
+	struct buf *buf = (struct buf *)rock->state;
+	buf_free(buf);
     }
     basic_free(rock);
 }
 
-void convert_free(struct convert_rock *rock) {
+static void convert_free(struct convert_rock *rock) {
     struct convert_rock *next;
     while (rock) {
 	next = rock->next;
@@ -655,7 +625,7 @@ void convert_free(struct convert_rock *rock) {
 
 /* converter initialisation routines */
 
-struct convert_rock *qp_init(int isheader, struct convert_rock *next) 
+static struct convert_rock *qp_init(int isheader, struct convert_rock *next)
 {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
     struct qp_state *s = xzmalloc(sizeof(struct qp_state));
@@ -666,7 +636,7 @@ struct convert_rock *qp_init(int isheader, struct convert_rock *next)
     return rock;
 }
 
-struct convert_rock *b64_init(struct convert_rock *next) 
+static struct convert_rock *b64_init(struct convert_rock *next)
 {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
     rock->state = xzmalloc(sizeof(struct b64_state));
@@ -675,7 +645,7 @@ struct convert_rock *b64_init(struct convert_rock *next)
     return rock;
 }
 
-struct convert_rock *stripnl_init(struct convert_rock *next)
+static struct convert_rock *stripnl_init(struct convert_rock *next)
 {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
     rock->f = stripnl2uni;
@@ -683,7 +653,7 @@ struct convert_rock *stripnl_init(struct convert_rock *next)
     return rock;
 }
 
-struct convert_rock *canon_init(int spacemode, struct convert_rock *next)
+static struct convert_rock *canon_init(int spacemode, struct convert_rock *next)
 {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
     struct canon_state *s = xzmalloc(sizeof(struct canon_state));
@@ -694,7 +664,7 @@ struct convert_rock *canon_init(int spacemode, struct convert_rock *next)
     return rock;
 }
 
-struct convert_rock *uni_init(struct convert_rock *next) 
+static struct convert_rock *uni_init(struct convert_rock *next)
 {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
     rock->f = uni2utf8;
@@ -702,7 +672,7 @@ struct convert_rock *uni_init(struct convert_rock *next)
     return rock;
 }
 
-struct convert_rock *table_init(int charset_num, struct convert_rock *next)
+static struct convert_rock *table_init(int charset_num, struct convert_rock *next)
 {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
     rock->state = xzmalloc(sizeof(struct table_state));
@@ -711,7 +681,7 @@ struct convert_rock *table_init(int charset_num, struct convert_rock *next)
     return rock;
 }
 
-struct convert_rock *search_init(const char *substr, comp_pat *pat) {
+static struct convert_rock *search_init(const char *substr, comp_pat *pat) {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
     struct search_state *s = xzmalloc(sizeof(struct search_state));
     struct comp_pat_s *p = (struct comp_pat_s *)pat;
@@ -723,7 +693,7 @@ struct convert_rock *search_init(const char *substr, comp_pat *pat) {
     s->substr = (unsigned char *)substr;
 
     /* allocate tracking space and initialise to "no match" */
-    s->starts = xmalloc(s->max_start * sizeof(s->starts[0]));
+    s->starts = xmalloc(s->max_start * sizeof(size_t));
     for (i = 0; i < s->max_start; i++) {
 	s->starts[i] = -1;
     }
@@ -736,13 +706,14 @@ struct convert_rock *search_init(const char *substr, comp_pat *pat) {
     return rock;
 }
 
-struct convert_rock *buffer_init(char *str, int len)
+static struct convert_rock *buffer_init(unsigned char *str, int len)
 {
     struct convert_rock *rock = xzmalloc(sizeof(struct convert_rock));
-    struct buffer_state *buf = xzmalloc(sizeof(struct buffer_state));
+    struct buf *buf = xzmalloc(sizeof(struct buf));
 
-    buf->base = str;
-    buf->alloced = len;
+    /* fiddling under the hood */
+    buf->s = (char *)str;
+    buf->alloc = len;
 
     rock->f = byte2buffer;
     rock->cleanup = buffer_free;
@@ -761,6 +732,15 @@ int charset_lookupname(const char *name)
 {
     int i;
 
+    /* translate to canonical name */
+    for (i = 0; charset_aliases[i].name; i++) {
+	if (!strcasecmp(name, charset_aliases[i].name)) {
+	    name = charset_aliases[i].canon_name;
+	    break;
+	}
+    }
+
+    /* look up canonical name */
     for (i = 0; i < chartables_num_charsets; i++) {
 	if (!strcasecmp(name, chartables_charset_table[i].name)) 
 	    return i;
@@ -769,9 +749,17 @@ int charset_lookupname(const char *name)
     return -1;
 }
 
+static int lookup_buf(const char *buf, int len)
+{
+    char *name = xstrndup(buf, len);
+    int res = charset_lookupname(name);
+    free(name);
+    return res;
+}
+
 /*
  * Convert the string 's' in the character set numbered 'charset'
- * into canonical searching form.  Decodes into 'retval', which 
+ * into canonical searching form.  Decodes into 'retval', which
  * must be reallocable and currently at least size 'alloced'.
  */
 char *charset_convert(const char *s, int charset, char *buf, size_t bufsz)
@@ -785,16 +773,16 @@ char *charset_convert(const char *s, int charset, char *buf, size_t bufsz)
 	return xstrdup("X");
 
     /* set up the conversion path */
-    tobuffer = buffer_init(buf, bufsz);
+    tobuffer = buffer_init((unsigned char *)buf, bufsz);
     input = uni_init(tobuffer);
-    input = canon_init(0, input);
+    input = canon_init(1, input);
     input = table_init(charset, input);
 
     /* do the conversion */
     convert_cat(input, s);
 
     /* extract the result */
-    res = buffer_cstring(tobuffer);
+    res = (char *)buffer_cstring(tobuffer);
 
     /* clean up */
     convert_free(input);
@@ -844,24 +832,22 @@ char *charset_to_utf8(const char *msg_base, size_t len, int charset, int encodin
     }
 
     convert_catn(input, msg_base, len);
-    res = buffer_cstring(tobuffer);
+    res = (char *)buffer_cstring(tobuffer);
     convert_free(input);
 
     return res;
 }
 
-void mimeheader_cat(struct convert_rock *target, const char *s)
+static void mimeheader_cat(struct convert_rock *target, const char *s)
 {
     struct convert_rock *input, *stripnl;
     int eatspace = 0;
     const char *start, *endcharset, *encoding, *end;
-    const char *p;
-    int i, c, c1, c2, c3, c4, charset;
-    int pos = 0;
     int len;
-    char *res;
+    int charset;
+    const char *p;
 
-    if (!s) return 0;
+    if (!s) return;
 
     /* set up the conversion path */
     input = table_init(0, target);
@@ -903,20 +889,16 @@ void mimeheader_cat(struct convert_rock *target, const char *s)
 	 * Get the 1522-word's character set
 	 */
 	start++;
-	for (charset = 0; charset < chartables_num_charsets; charset++) {
-	    if ((int)strlen(chartables_charset_table[charset].name) == endcharset-start &&
-		!strncasecmp(start, chartables_charset_table[charset].name, endcharset-start)) {
-		table_switch(input, charset);
-		break;
-	    }
-	}
-
-	if (charset == chartables_num_charsets) {
+	charset = lookup_buf(start, endcharset-start);
+	if (charset < 0) {
 	    /* Unrecognized charset, nothing will match here */
 	    convert_putc(input, 0xfffd); /* unknown character */
 	}
 	else {
 	    struct convert_rock *extract;
+
+	    table_switch(input, charset);
+
 	    /* choose decoder */
 	    if (encoding[1] == 'q' || encoding[1] == 'Q') {
 		extract = qp_init(1, input);
@@ -957,13 +939,13 @@ char *charset_decode_mimeheader(const char *s, char *retval, size_t alloced)
     struct convert_rock *tobuffer, *input;
     char *res;
 
-    tobuffer = buffer_init(retval, alloced);
+    tobuffer = buffer_init((unsigned char *)retval, alloced);
     input = uni_init(tobuffer);
-    input = canon_init(0, input);
+    input = canon_init(1, input);
 
     mimeheader_cat(input, s);
  
-    res = buffer_cstring(tobuffer);
+    res = (char *)buffer_cstring(tobuffer);
 
     convert_free(input);
 
@@ -985,7 +967,7 @@ char *charset_parse_mimeheader(const char *s)
 
     mimeheader_cat(input, s);
  
-    res = buffer_cstring(tobuffer);
+    res = (char *)buffer_cstring(tobuffer);
 
     convert_free(input);
 
@@ -1000,7 +982,7 @@ int charset_search_mimeheader(const char *substr, comp_pat *pat,
 
     tosearch = search_init(substr, pat);
     input = uni_init(tosearch);
-    if (searchform) input = canon_init(0, input);
+    if (searchform) input = canon_init(1, input);
 
     mimeheader_cat(input, s);
  
@@ -1049,7 +1031,7 @@ int charset_searchstring(const char *substr, comp_pat *pat,
     const char *s, size_t len)
 {
     struct convert_rock *tosearch;
-    int res, *found;
+    int res;
 
     /* set up the search handler */
     tosearch = search_init(substr, pat);
@@ -1071,18 +1053,18 @@ int charset_searchstring(const char *substr, comp_pat *pat,
 
 /*
  * Search for the string 'substr' in the next 'len' bytes of 
- * 'msg_base'.  If 'mapnl' is nonzero, then LF characters in the file
- * map to CR LF and count as 2 bytes w.r.t. the value of 'len'.
+ * 'msg_base'.  
  * 'charset' and 'encoding' specify the character set and 
  * content transfer encoding of the data, respectively.
  * Returns nonzero iff the string was found.
  */
 int charset_searchfile(const char *substr, comp_pat *pat,
-    const char *msg_base, int mapnl, size_t len, int charset, 
+    const char *msg_base, size_t len, int charset,
     int encoding)
 {
     struct convert_rock *input, *tosearch;
-    int i, *found, res;
+    size_t i;
+    int res;
 
     /* Initialize character set mapping */
     if (charset < 0 || charset >= chartables_num_charsets) 
@@ -1095,7 +1077,7 @@ int charset_searchfile(const char *substr, comp_pat *pat,
     /* set up the conversion path */
     tosearch = search_init(substr, pat);
     input = uni_init(tosearch);
-    input = canon_init(0, input);
+    input = canon_init(1, input);
     input = table_init(charset, input);
 
     /* choose encoding extraction if needed */
@@ -1122,10 +1104,6 @@ int charset_searchfile(const char *substr, comp_pat *pat,
 
     /* implement the loop here so we can check on the search each time */
     for (i = 0; i < len; i++) {
-	if (mapnl && msg_base[i] == '\n') {
-	    convert_putc(input, '\r');
-	    len--;
-	}
 	convert_putc(input, msg_base[i]);
 	if (search_havematch(tosearch)) break;
     }
@@ -1139,23 +1117,23 @@ int charset_searchfile(const char *substr, comp_pat *pat,
 
 /* This is based on charset_searchfile above. */
 int charset_extractfile(index_search_text_receiver_t receiver,
-    void* rock, int uid, const char *msg_base, int mapnl, size_t len, 
+    void* rock, int uid, const char *msg_base, size_t len,
     int charset, int encoding)
 {
     struct convert_rock *input, *tobuffer;
-    struct buffer_state *out;
-    int i;
-
-    /* set up the conversion path */
-    tobuffer = buffer_init(0, 0);
-    input = uni_init(tobuffer);
-    input = canon_init(0, input);
-    input = table_init(charset, input);
+    struct buf *out;
+    size_t i;
 
     /* Initialize character set mapping */
     if (charset < 0 || charset >= chartables_num_charsets) 
 	return 0;
 
+    /* set up the conversion path */
+    tobuffer = buffer_init(0, 0);
+    input = uni_init(tobuffer);
+    input = canon_init(1, input);
+    input = table_init(charset, input);
+
     switch (encoding) {
     case ENCODING_NONE:
 	break;
@@ -1178,23 +1156,21 @@ int charset_extractfile(index_search_text_receiver_t receiver,
     }
 
     /* point to the buffer for easy block sending */
-    out = (struct buffer_state *)tobuffer->state;
+    out = (struct buf *)tobuffer->state;
 
     for (i = 0; i < len; i++) {
-	if (mapnl && msg_base[i] == '\n') {
-	    convert_putc(input, '\r');
-	    len--;
-	}
 	convert_putc(input, msg_base[i]);
 
 	/* process a block of output every so often */
-	if (out->offset > 4096) {
-	    receiver(uid, SEARCHINDEX_PART_BODY, SEARCHINDEX_CMD_APPENDPART, out->base, out->offset, rock);
-	    out->offset = 0;
+	if (buf_len(out) > 4096) {
+	    receiver(uid, SEARCHINDEX_PART_BODY, SEARCHINDEX_CMD_APPENDPART,
+		     out->s, out->len, rock);
+	    buf_reset(out);
 	}
     }
-    if (out->offset) { /* finish it */
-	receiver(uid, SEARCHINDEX_PART_BODY, SEARCHINDEX_CMD_APPENDPART, out->base, out->offset, rock);
+    if (out->len) { /* finish it */
+	receiver(uid, SEARCHINDEX_PART_BODY, SEARCHINDEX_CMD_APPENDPART,
+		 out->s, out->len, rock);
     }
 
     convert_free(input);
@@ -1207,7 +1183,7 @@ int charset_extractfile(index_search_text_receiver_t receiver,
  * 'msg_base' having the content transfer 'encoding'.  Decodes into
  * 'retval' (if necessary), which must be reallocable and currently at
  * least size 'alloced'.  Returns the number of decoded bytes in
- * 'outlen'. 
+ * 'outlen'.
  */
 char *charset_decode_mimebody(const char *msg_base, size_t len, int encoding,
 			      char **retval, size_t alloced, size_t *outlen)
@@ -1220,12 +1196,12 @@ char *charset_decode_mimebody(const char *msg_base, size_t len, int encoding,
 	return (char *) msg_base;
 
     case ENCODING_QP:
-	tobuffer = buffer_init(*retval, alloced);
+	tobuffer = buffer_init((unsigned char *)*retval, alloced);
 	input = qp_init(0, tobuffer);
 	break;
 
     case ENCODING_BASE64:
-	tobuffer = buffer_init(*retval, alloced);
+	tobuffer = buffer_init((unsigned char *)*retval, alloced);
 	input = b64_init(tobuffer);
 	break;
 
@@ -1239,12 +1215,12 @@ char *charset_decode_mimebody(const char *msg_base, size_t len, int encoding,
     /* extract the string from the buffer, messy - but we want to
      * do it without becoming a cstring or being prematurely freed! */
     {
-	struct buffer_state *buf = (struct buffer_state *)tobuffer->state;
-	*retval = buf->base;
-	*outlen = buf->offset;
-	buf->base = 0;
-	buf->alloced = 0;
-	buf->offset = 0;
+	struct buf *buf = (struct buf *)tobuffer->state;
+	*retval = buf->s;
+	*outlen = buf->len;
+	buf->s = 0;
+	buf->len = 0;
+	buf->alloc = 0;
     }
 
     convert_free(input);
