@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: cyrusdb_sql.c,v 1.2 2009/03/31 04:11:22 brong Exp $ */
+/* $Id: cyrusdb_sql.c,v 1.3 2010/01/06 17:01:45 murch Exp $ */
 
 #include <config.h>
 
@@ -236,12 +236,13 @@ static void *_pgsql_open(char *host, char *port, int usessl,
 }
 
 static char *_pgsql_escape(void *conn __attribute__((unused)),
-			   char **to, const char *from, size_t fromlen)
+			   char **to __attribute__((unused)),
+			   const char *from, size_t fromlen)
 {
     size_t tolen;
 
     /* returned buffer MUST be freed by caller */
-    return PQescapeBytea((char *) from, fromlen, &tolen);
+    return (char *) PQescapeBytea((unsigned char *) from, fromlen, &tolen);
 }
 
 static int _pgsql_exec(void *conn, const char *cmd, exec_cb *cb, void *rock)
@@ -274,8 +275,12 @@ static int _pgsql_exec(void *conn, const char *cmd, exec_cb *cb, void *rock)
 	char *key, *data;
 	size_t keylen, datalen;
 
-	key = PQunescapeBytea(PQgetvalue(result, i, 0), &keylen);
-	data = PQunescapeBytea(PQgetvalue(result, i, 1), &datalen);
+	key = (char *)
+	    PQunescapeBytea((unsigned char *) PQgetvalue(result, i, 0),
+			    &keylen);
+	data = (char *)
+	    PQunescapeBytea((unsigned char *) PQgetvalue(result, i, 1),
+			    &datalen);
 	r = cb(rock, key, keylen, data, datalen);
 	free(key); free(data);
     }
@@ -665,9 +670,8 @@ static int fetch(struct db *db,
     esc_key = dbengine->sql_escape(db->conn, &db->esc_key, key, keylen);
     snprintf(cmd, sizeof(cmd),
 	     "SELECT * FROM %s WHERE dbkey = '%s';", db->table, esc_key);
-    r = dbengine->sql_exec(db->conn, cmd, &select_cb, &srock);
-
     if (esc_key != db->esc_key) free(esc_key);
+    r = dbengine->sql_exec(db->conn, cmd, &select_cb, &srock);
 
     if (r) {
 	syslog(LOG_ERR, "DBERROR: SQL failed %s", cmd);
@@ -704,10 +708,9 @@ static int foreach(struct db *db,
     snprintf(cmd, sizeof(cmd),
 	     "SELECT * FROM %s WHERE dbkey LIKE '%s%%' ORDER BY dbkey;",
 	     db->table, esc_key ? esc_key : "");
+    if (esc_key && (esc_key != db->esc_key)) free(esc_key);
     r = dbengine->sql_exec(db->conn, cmd, &select_cb, &srock);
 
-    if (esc_key && esc_key != db->esc_key) free(esc_key);
-
     if (r) {
 	syslog(LOG_ERR, "DBERROR: SQL failed %s", cmd);
 	if (tid) dbengine->sql_rollback_txn(db->conn);
@@ -723,11 +726,14 @@ static int mystore(struct db *db,
 		   struct txn **tid, int overwrite)
 {
     char cmd[1024], *esc_key;
+    int free_esc_key = 0;
+    const char dummy = 0;
     int r = 0;
 
     if (tid && !*tid && !(*tid = start_txn(db))) return CYRUSDB_INTERNAL;
 
     esc_key = dbengine->sql_escape(db->conn, &db->esc_key, key, keylen);
+    free_esc_key = (esc_key != db->esc_key);
 
     if (!data) {
 	/* DELETE the entry */
@@ -748,6 +754,7 @@ static int mystore(struct db *db,
 
 	char *esc_data = dbengine->sql_escape(db->conn, &db->esc_data,
 					      data, datalen);
+	int free_esc_data = (esc_data != db->esc_data);
 
 	/* see if we just SELECTed this key in this transaction */
 	if (tid && *tid) {
@@ -787,10 +794,10 @@ static int mystore(struct db *db,
 	    r = dbengine->sql_exec(db->conn, cmd, NULL, NULL);
 	}
 
-	if (esc_data != db->esc_data) free(esc_data);
+	if (free_esc_data) free(esc_data);
     }
 
-    if (esc_key != db->esc_key) free(esc_key);
+    if (free_esc_key) free(esc_key);
 
     if (r) {
 	syslog(LOG_ERR, "DBERROR: SQL failed: %s", cmd);
