@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: libconfig.c,v 1.24 2009/08/20 15:26:15 murch Exp $
+ * $Id: libconfig.c,v 1.26 2010/04/19 19:54:26 murch Exp $
  */
 
 #include <config.h>
@@ -80,8 +80,10 @@ const char *config_ident = NULL;         /* the service name */
 int config_hashimapspool;	  /* f */
 enum enum_value config_virtdomains;	          /* f */
 enum enum_value config_mupdate_config;	/* IMAP_ENUM_MUPDATE_CONFIG_STANDARD */
-int config_maxword;
-int config_maxquoted;
+int config_auditlog;
+unsigned config_maxword;
+unsigned config_maxquoted;
+int config_qosmarking;
 
 /* declared in each binary that uses libconfig */
 extern const int config_need_data;
@@ -107,7 +109,7 @@ int config_getint(enum imapopt opt)
 #if (SIZEOF_LONG != 4)
     if ((imapopts[opt].val.i > 0x7fffffff)||
 	(imapopts[opt].val.i < -0x7fffffff)) {
-	syslog(LOG_ERR, "config_getint: %s: %lld too large for type",
+	syslog(LOG_ERR, "config_getint: %s: %ld too large for type",
 	       imapopts[opt].optname, imapopts[opt].val.i);
     }
 #endif    
@@ -121,7 +123,7 @@ int config_getswitch(enum imapopt opt)
 #if (SIZEOF_LONG != 4)
     if ((imapopts[opt].val.b > 0x7fffffff)||
 	(imapopts[opt].val.b < -0x7fffffff)) {
-	syslog(LOG_ERR, "config_getswitch: %s: %lld too large for type", 
+	syslog(LOG_ERR, "config_getswitch: %s: %ld too large for type", 
 	       imapopts[opt].optname, imapopts[opt].val.b);
     }
 #endif    
@@ -157,7 +159,8 @@ const char *config_getoverflowstring(const char *key, const char *def)
     if(config_ident) {
 	if(snprintf(buf,sizeof(buf),"%s_%s",config_ident,key) == -1)
 	    fatal("key too long in config_getoverflowstring", EC_TEMPFAIL);
-    
+
+	lcase(buf);
 	ret = hash_lookup(buf, &confighash);
     }
     
@@ -210,11 +213,21 @@ static void config_ispartition(const char *key,
     if (!strncmp("partition-", key, 10)) *found = 1;
 }
 
+static const unsigned char qos[] = {
+/* cs0..cs7 */		0x00, 0x20, 0x40, 0x60, 0x80, 0xa0, 0xc0, 0xe0,
+/* af11..af13 */	0x28, 0x30, 0x38,
+/* af21..af23 */	0x48, 0x50, 0x58,
+/* af31..af33 */	0x68, 0x70, 0x78,
+/* af41..af43 */	0x88, 0x90, 0x98,
+/* ef */		0xb8
+};
+
 void config_read(const char *alt_config)
 {
     enum imapopt opt = IMAPOPT_ZERO;
     char buf[4096];
     char *p;
+    int ival;
 
     /* xxx this is leaked, this may be able to be better in 2.2 (cyrus_done) */
     if(alt_config) config_filename = xstrdup(alt_config);
@@ -284,6 +297,12 @@ void config_read(const char *alt_config)
 	if (Uisupper(*p)) *p = tolower((unsigned char) *p);
     }
 
+    config_mupdate_server = config_getstring(IMAPOPT_MUPDATE_SERVER);
+
+    if (config_mupdate_server) {
+	config_mupdate_config = config_getenum(IMAPOPT_MUPDATE_CONFIG);
+    }
+
     if (config_need_data & CONFIG_NEED_PARTITION_DATA) {
 	int found = 0;
 
@@ -291,6 +310,10 @@ void config_read(const char *alt_config)
 	    /* see if defaultpartition is specified properly */
 	    if (config_partitiondir(config_defpartition)) found = 1;
 	}
+	else if ((config_mupdate_config == IMAP_ENUM_MUPDATE_CONFIG_STANDARD)
+		 && !config_getstring(IMAPOPT_PROXYSERVERS)) {
+	    found = 1; /* don't need partitions on the frontend */
+	}
 	else {
 	    /* see if we have ANY partition-<name> options */
 	    config_foreachoverflowstring(config_ispartition, &found);
@@ -311,6 +334,9 @@ void config_read(const char *alt_config)
     config_virtdomains = config_getenum(IMAPOPT_VIRTDOMAINS);
     config_defdomain = config_getstring(IMAPOPT_DEFAULTDOMAIN);
 
+    /* are we auditlogging */
+    config_auditlog = config_getswitch(IMAPOPT_AUDITLOG);
+
     /* look up the hostname and info we should present to the user */
     config_servername = config_getstring(IMAPOPT_SERVERNAME);
     if (!config_servername) {
@@ -319,25 +345,23 @@ void config_read(const char *alt_config)
     }
     config_serverinfo = config_getenum(IMAPOPT_SERVERINFO);
 
-    config_mupdate_server = config_getstring(IMAPOPT_MUPDATE_SERVER);
-
-    if (config_mupdate_server) {
-	config_mupdate_config = config_getenum(IMAPOPT_MUPDATE_CONFIG);
-    }
-
     /* set some limits */
     config_maxquoted = config_getint(IMAPOPT_MAXQUOTED);
     config_maxword = config_getint(IMAPOPT_MAXWORD);
+
+    ival = config_getenum(IMAPOPT_QOSMARKING);
+    config_qosmarking = qos[ival];
 }
 
 #define GROWSIZE 4096
 
 void config_read_file(const char *filename)
 {
-    FILE *infile;
+    FILE *infile = NULL;
     enum imapopt opt = IMAPOPT_ZERO;
     int lineno = 0;
     char *buf, errbuf[1024];
+    const char *cyrus_path;
     unsigned bufsize, len;
     char *p, *q, *key, *fullkey, *srvkey, *val, *newval;
     int service_specific;
@@ -346,16 +370,21 @@ void config_read_file(const char *filename)
     bufsize = GROWSIZE;
     buf = xmalloc(bufsize);
 
-    /* read in config file */
-    infile = fopen(filename, "r");
-    if (!infile) {
-	strlcpy(buf, CYRUS_PATH, bufsize);
+    /* read in config file
+       Check if we have CYRUS_PREFIX defined, and then use that config */
+    cyrus_path = getenv("CYRUS_PREFIX");
+    if (cyrus_path) {
+	strlcpy(buf, cyrus_path, bufsize);
 	strlcat(buf, filename, bufsize);
 	infile = fopen(buf, "r");
     }
+
+    if (!infile)
+	infile = fopen(filename, "r");
+
     if (!infile) {
 	snprintf(buf, bufsize, "can't open configuration file %s: %s",
-		 filename, error_message(errno));
+		 filename, strerror(errno));
 	fatal(buf, EC_CONFIG);
     }
 
