@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: nonblock_fcntl.c,v 1.16 2008/03/24 17:43:09 murch Exp $
  */
 
 #include <config.h>
@@ -68,10 +66,7 @@ const char *nonblock_method_desc = "fcntl";
  * 'mode' is nonzero, sets non-blocking mode, if 'mode' is zero
  * clears non-blocking mode.
  */
-void
-nonblock(fd, mode)
-int fd;
-int mode;
+void nonblock(int fd, int mode)
 {
     int flags;
 
@@ -83,5 +78,5 @@ int mode;
     else {
 	flags &= ~NON_BLOCKING_MODE;
     }
-    fcntl(fd, F_SETFL, flags);
+    (void)fcntl(fd, F_SETFL, flags);
 }
