@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: nonblock_ioctl.c,v 1.13 2010/01/06 17:01:46 murch Exp $
  */
 
 #include <config.h>
@@ -58,10 +56,7 @@ const char *nonblock_method_desc = "ioctl";
  * 'mode' is nonzero, sets non-blocking mode, if 'mode' is zero
  * clears non-blocking mode.
  */
-void
-nonblock(fd, mode)
-int fd;
-int mode;
+void nonblock(int fd, int mode)
 {
     mode = mode ? 1 : 0;
 
