@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: prot.c,v 1.100 2010/06/28 12:06:43 brong Exp $
  */
 
 #include <config.h>
@@ -228,9 +226,7 @@ int prot_sasldecode(struct protstream *s, int n)
  * Turn on SASL for this connection
  */
 
-int prot_setsasl(s, conn)
-struct protstream *s;
-sasl_conn_t *conn;
+int prot_setsasl(struct protstream *s, sasl_conn_t *conn)
 {
     const void *ssfp;
     int result;
@@ -1277,6 +1273,9 @@ int prot_printf(struct protstream *s, const char *fmt, ...)
 	    prot_write(s, buf, strlen(buf));
 	    break;
 
+	/* according to linux 'man 3 printf' this is supposed to be
+	 * of type "ptrdiff_t", not "size_t" - but don't want to break
+	 * running code! */
 	case 't': {
 	    size_t tu;
 	    ssize_t td;
@@ -1310,6 +1309,29 @@ int prot_printf(struct protstream *s, const char *fmt, ...)
 	    (void)prot_putc(i, s);
 	    break;
 
+	case 'z': {
+	    size_t zu;
+	    ssize_t zd;
+
+	    switch (*++percent) {
+	    case 'u':
+		zu = va_arg(pvar, size_t);
+		snprintf(buf, sizeof(buf), "%zu", zu);
+		prot_write(s, buf, strlen(buf));
+		break;
+
+	    case 'd':
+		zd = va_arg(pvar, ssize_t);
+		snprintf(buf, sizeof(buf), "%zd", zd);
+		prot_write(s, buf, strlen(buf));
+		break;
+
+	    default:
+		abort();
+	    }
+	    break;
+	}
+
 	default:
 	    abort();
 	}
