@@ -1,12 +1,12 @@
-# 
-# Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+#
+# Copyright (c) 1994-2008 Carnegie Mellon University.  All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
 #
 # 1. Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer. 
+#    notice, this list of conditions and the following disclaimer.
 #
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
@@ -15,14 +15,15 @@
 #
 # 3. The name "Carnegie Mellon University" must not be used to
 #    endorse or promote products derived from this software without
-#    prior written permission. For permission or any other legal
-#    details, please contact  
-#      Office of Technology Transfer
+#    prior written permission. For permission or any legal
+#    details, please contact
 #      Carnegie Mellon University
-#      5000 Forbes Avenue
-#      Pittsburgh, PA  15213-3890
-#      (412) 268-4387, fax: (412) 268-7395
-#      tech-transfer@andrew.cmu.edu
+#      Center for Technology Transfer and Enterprise Creation
+#      4615 Forbes Avenue
+#      Suite 302
+#      Pittsburgh, PA  15213
+#      (412) 268-7393, fax: (412) 268-7395
+#      innovation@andrew.cmu.edu
 #
 # 4. Redistributions of any form whatsoever must retain the following
 #    acknowledgment:
@@ -37,7 +38,7 @@
 # AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 # OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
-# $Id: Shell.pm,v 1.41 2007/10/02 01:49:20 jeaton Exp $
+# $Id: Shell.pm,v 1.44 2010/01/06 17:01:55 murch Exp $
 #
 # A shell framework for Cyrus::IMAP::Admin
 #
@@ -126,7 +127,7 @@ my %builtins = (exit =>
 		  [\&_sc_info, '[mailbox]',
 		   'display mailbox/server metadata'],
 		mboxcfg =>
-		  [\&_sc_mboxcfg, 'mailbox [comment|condstore|expire|news2mail|sharedseen|sieve|squat] value',
+		  [\&_sc_mboxcfg, 'mailbox [comment|expire|news2mail|sieve|squat|/<explicit annotation>] value',
 		   'configure mailbox'],
 		mboxconfig => 'mboxcfg',
 		reconstruct =>
@@ -1176,6 +1177,18 @@ sub _sc_deleteacl {
       my $name = $mbx->[0];
       my $flags = $mbx->[1];
       next if($flags =~ /(\\noselect|\\nonexistent|\\placeholder)/i);
+      # If id of '*' is passed then delete all existing acls
+      if ($nargv[1] eq '*') {
+        my %acl = $$cyrref->listaclmailbox($name);
+        if (defined $$cyrref->error) {
+          print($$cyrref->error, "\n");
+          next;
+        }
+        pop(@nargv);
+        foreach my $acl (keys %acl) {
+          push(@nargv, $acl, $acl{$acl});
+        }
+      }
       print "Deleting acl on $name...";
       $nargv[0] = $name;
       my $rc = $$cyrref->deleteacl(@nargv);
@@ -1187,6 +1200,18 @@ sub _sc_deleteacl {
       }
     }
   } else {
+    # If id of '*' is passed then delete all existing acls
+    if ($nargv[1] eq '*') {
+      my %acl = $$cyrref->listaclmailbox($nargv[0]);
+      if (defined $$cyrref->error) {
+        print($$cyrref->error, "\n");
+        next;
+      }
+      pop(@nargv);
+      foreach my $acl (keys %acl) {
+        push(@nargv, $acl, $acl{$acl});
+      }
+    }
     $$cyrref->deleteacl(@nargv) ||
       die "deleteaclmailbox: " . $$cyrref->error . "\n";
   }
@@ -1436,7 +1461,7 @@ sub _sc_mboxcfg {
   while (defined ($opt = shift(@argv))) {
     last if $opt eq '--';
     if ($opt =~ /^-/) {
-      die "usage: mboxconfig mailbox [comment|condstore|expire|news2mail|sharedseen|sieve|squat] value\n";
+      die "usage: mboxconfig mailbox [comment|expire|news2mail|sharedseen|sieve|squat|/<explicit annotation>] value\n";
     }
     else {
       push(@nargv, $opt);
@@ -1445,7 +1470,7 @@ sub _sc_mboxcfg {
   }
   push(@nargv, @argv);
   if (@nargv < 2) {
-    die "usage: mboxconfig mailbox [comment|condstore|expire|news2mail|sharedseen|sieve|squat] value\n";
+    die "usage: mboxconfig mailbox [comment|expire|news2mail|sharedseen|sieve|squat|/<explicit annotation>] value\n";
   }
   if (!$cyrref || !$$cyrref) {
     die "mboxconfig: no connection to server\n";
@@ -1675,10 +1700,6 @@ The currently supported attributes are:
 
 Sets a comment or description associated with the mailbox.
 
-=item C<condstore>
-
-Enables the IMAP CONDSTORE extension (modification sequences) on the mailbox.
-
 =item C<expire>
 
 Sets the number of days after which messages will be expired from the mailbox.
@@ -1807,8 +1828,8 @@ The currently supported attributes are:
 
 =item C<motd>
 
-Sets a "message of the day".  The message gets displayed as an ALERT after
-authentication.
+Sets a "message of the day".  The message gets displayed as an ALERT upon
+connection.
 
 =item C<comment>
 
