@@ -41,7 +41,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: lex.c,v 1.10 2008/04/04 12:47:20 murch Exp $
+ * $Id: lex.c,v 1.11 2010/01/06 17:01:56 murch Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -82,7 +82,16 @@ int token_lookup(char *str, int len __attribute__((unused)))
   if (strcmp(str,"active")==0) return TOKEN_ACTIVE;
   if (strcmp(str,"referral")==0) return TOKEN_REFERRAL;
   if (strcmp(str,"sasl")==0) return TOKEN_SASL;
-  
+  if (strcmp(str,"quota/maxscripts")==0) return RESP_CODE_QUOTA_MAXSCRIPTS;
+  if (strcmp(str,"quota/maxsize")==0) return RESP_CODE_QUOTA_MAXSIZE;
+  if (strcmp(str,"quota")==0) return RESP_CODE_QUOTA;
+  if (strcmp(str,"transition-needed")==0) return RESP_CODE_TRANSITION_NEEDED;
+  if (strcmp(str,"trylater")==0) return RESP_CODE_TRYLATER;
+  if (strcmp(str,"nonexistent")==0) return RESP_CODE_NONEXISTENT;
+  if (strcmp(str,"alreadyexists")==0) return RESP_CODE_ALREADYEXISTS;
+  if (strcmp(str,"warning")==0) return RESP_CODE_WARNINGS;
+  if (strcmp(str,"tag")==0) return RESP_CODE_TAG;
+
   return -1;
 }
 
@@ -267,7 +276,7 @@ int yylex(lexstate_t * lvalp, void * client)
       }
       break;
     case LEXER_STATE_ATOM:
-      if (!isalpha((unsigned char) ch)) {
+      if (!(isalpha((unsigned char) ch) || ch == '/')) {
 	int token;
 
 	buffer[ buff_ptr - buffer] = '\0';
