@@ -117,7 +117,7 @@ int handle_response(int res,int version,struct protstream *pin,
 		  res = yylex(&state, pin);
 	      }
 	      if(res != ')') {
-		  parseerror("expected RPARAN");
+		  parseerror("expected RPAREN");
 	      }
 	  }
 
@@ -147,7 +147,7 @@ int handle_response(int res,int version,struct protstream *pin,
 	      res = yylex(&state, pin);
 	  }
 	  if(res != ')') {
-	      parseerror("expected RPARAN");
+	      parseerror("expected RPAREN");
 	  }
 	  
 	  res = yylex(&state, pin);
@@ -547,9 +547,7 @@ static int writefile(mystring_t *data, char *name, char **errstrp)
 
   char *scrname;
 
-  scrname=malloc(strlen(name)+10);
-  strcpy(scrname, name);
-  strcat(scrname, ".script");
+  scrname = strconcat(name, ".script", (char *)NULL);
 
   stream=fopen(scrname,"w");
 
