@@ -41,7 +41,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: bc_emit.c,v 1.4 2008/03/24 20:08:46 murch Exp $
+ * $Id: bc_emit.c,v 1.5 2010/01/06 17:01:58 murch Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,7 +60,7 @@
 
 
 #if DUMPCODE
-void dump(bytecode_info_t *d);
+void dump(bytecode_info_t *d, int level);
 #endif
 
 static inline int write_int (int fd, int x)
@@ -87,7 +87,7 @@ static int align_string(int fd, int string_len)
     int needed = sizeof(int) - (string_len % sizeof(int));
     if (needed>= 0 && needed <=4)
     {
-    	if(write(fd, "\0\0\0\0", needed) == -1) return -1;
+	if (write(fd, "\0\0\0\0", needed) == -1) return -1;
     }
     return needed;
 }
@@ -139,6 +139,7 @@ static int bc_stringlist_emit(int fd, int *codep, bytecode_info_t *bc)
 	wrote+=ret;
     }
     end=lseek(fd,0,SEEK_CUR);
+    if (end < 0) return -1;
  
     /* go back and write end of list position */
     lseek(fd,begin,SEEK_SET);
@@ -180,6 +181,7 @@ static int bc_testlist_emit(int fd, int *codep, bytecode_info_t *bc)
 	*codep = nextcodep;
     }
     end = lseek(fd, 0, SEEK_CUR);
+    if (end < 0) return -1;
 
     /* go back and write the end of list position */
     lseek(fd,begin,SEEK_SET);
@@ -254,6 +256,7 @@ static int bc_test_emit(int fd, int *codep, bytecode_info_t *bc)
     }
     
     case BC_HEADER:
+    case BC_HASFLAG:
     {
 	int ret;
 	/* Drop match type */
@@ -261,21 +264,21 @@ static int bc_test_emit(int fd, int *codep, bytecode_info_t *bc)
 	    return -1;
 	wrote += sizeof(int);
 	(*codep)++;
-	/*drop comparator */
-	if(write_int(fd, bc->data[(*codep)].value) == -1)
-	    return -1;
-	wrote += sizeof(int);
-	(*codep)++;    
 	/*now drop relation*/
 	if(write_int(fd, bc->data[(*codep)].value) == -1)
 	    return -1;
 	wrote += sizeof(int);
+	(*codep)++;    
+	/*drop comparator */
+	if(write_int(fd, bc->data[(*codep)].value) == -1)
+	    return -1;
+	wrote += sizeof(int);
 	(*codep)++;
-	/* Now drop headers */
+	/* Now drop haystacks */
 	ret = bc_stringlist_emit(fd, codep, bc);
 	if(ret < 0) return -1;
 	wrote+=ret;
-	/* Now drop data */
+	/* Now drop needles */
 	ret = bc_stringlist_emit(fd, codep, bc);
 	if(ret < 0) return -1;
 	wrote+=ret;
@@ -510,9 +513,57 @@ static int bc_action_emit(int fd, int codep, int stopcodep,
 	    break;
 	}
 	
+	case B_KEEP:
+	    /* Flags Stringlist, Copy (word) */
+
+	    /* Dump a stringlist of flags */
+	    ret = bc_stringlist_emit(fd, &codep, bc);
+	    if(ret < 0)
+		return -1;
+	    filelen += ret;
+
+	    if(write_int(fd,bc->data[codep++].value) == -1)
+		return -1;
+
+	    filelen += sizeof(int);
+	    break;
+
 	case B_FILEINTO:
+	    /* Flags Stringlist, Copy (word), Folder String */
+
+	    /* Dump a stringlist of flags */
+	    ret = bc_stringlist_emit(fd, &codep, bc);
+	    if(ret < 0)
+		return -1;
+	    filelen += ret;
+
+	    /* Write Copy */
+	    if(write_int(fd,bc->data[codep++].value) == -1)
+		return -1;
+
+	    filelen += sizeof(int);
+
+	    /* Write string length of Folder */
+	    len = bc->data[codep++].len;
+	    if(write_int(fd,len) == -1)
+		return -1;
+
+	    filelen+=sizeof(int);
+
+	    /* Write Folder */
+	    if(write(fd,bc->data[codep++].str,len) == -1)
+		return -1;
+
+	    ret = align_string(fd, len);
+	    if(ret == -1)
+		return -1;
+
+	    filelen += len + ret;
+
+	    break;
+
 	case B_REDIRECT:
-	    /* Copy (word), Folder/Address String */
+	    /* Copy (word), Address String */
 
 	    if(write_int(fd,bc->data[codep++].value) == -1)
 		return -1;
@@ -770,7 +821,6 @@ static int bc_action_emit(int fd, int codep, int stopcodep,
 	case B_NULL:
 	case B_STOP:
 	case B_DISCARD:
-	case B_KEEP:
 	case B_MARK:
 	case B_UNMARK:
 	case B_RETURN:
@@ -798,7 +848,7 @@ int sieve_emit_bytecode(int fd, bytecode_info_t *bc)
     if(write_int(fd, data) == -1) return -1;
 
 #if DUMPCODE
-    dump(bc);
+    dump(bc, 0);
 #endif
 
     /*the sizeof(int) is to account for the version # at the begining*/
