@@ -40,7 +40,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: script.h,v 1.13 2008/03/24 20:08:46 murch Exp $
+ * $Id: script.h,v 1.14 2010/01/06 17:01:59 murch Exp $
  */
 
 #ifndef SIEVE_SCRIPT_H
@@ -52,8 +52,6 @@
 #include "interp.h"
 #include "tree.h"
 
-#define ADDRERR_SIZE 500
-
 struct sieve_script {
     sieve_interp_t interp;
 
@@ -72,6 +70,7 @@ struct sieve_script {
 	int i_ascii_numeric: 1;
 	int include        : 1;
 	int copy           : 1;
+	int imap4flags     : 1;
     } support;
 
     void *script_context;
