@@ -55,61 +55,63 @@
      REMOVEFLAG = 273,
      MARK = 274,
      UNMARK = 275,
-     NOTIFY = 276,
-     DENOTIFY = 277,
-     ANYOF = 278,
-     ALLOF = 279,
-     EXISTS = 280,
-     SFALSE = 281,
-     STRUE = 282,
-     HEADER = 283,
-     NOT = 284,
-     SIZE = 285,
-     ADDRESS = 286,
-     ENVELOPE = 287,
-     BODY = 288,
-     COMPARATOR = 289,
-     IS = 290,
-     CONTAINS = 291,
-     MATCHES = 292,
-     REGEX = 293,
-     COUNT = 294,
-     VALUE = 295,
-     OVER = 296,
-     UNDER = 297,
-     GT = 298,
-     GE = 299,
-     LT = 300,
-     LE = 301,
-     EQ = 302,
-     NE = 303,
-     ALL = 304,
-     LOCALPART = 305,
-     DOMAIN = 306,
-     USER = 307,
-     DETAIL = 308,
-     RAW = 309,
-     TEXT = 310,
-     CONTENT = 311,
-     DAYS = 312,
-     ADDRESSES = 313,
-     SUBJECT = 314,
-     FROM = 315,
-     HANDLE = 316,
-     MIME = 317,
-     METHOD = 318,
-     ID = 319,
-     OPTIONS = 320,
-     LOW = 321,
-     NORMAL = 322,
-     HIGH = 323,
-     ANY = 324,
-     MESSAGE = 325,
-     INCLUDE = 326,
-     PERSONAL = 327,
-     GLOBAL = 328,
-     RETURN = 329,
-     COPY = 330
+     HASFLAG = 276,
+     FLAGS = 277,
+     NOTIFY = 278,
+     DENOTIFY = 279,
+     ANYOF = 280,
+     ALLOF = 281,
+     EXISTS = 282,
+     SFALSE = 283,
+     STRUE = 284,
+     HEADER = 285,
+     NOT = 286,
+     SIZE = 287,
+     ADDRESS = 288,
+     ENVELOPE = 289,
+     BODY = 290,
+     COMPARATOR = 291,
+     IS = 292,
+     CONTAINS = 293,
+     MATCHES = 294,
+     REGEX = 295,
+     COUNT = 296,
+     VALUE = 297,
+     OVER = 298,
+     UNDER = 299,
+     GT = 300,
+     GE = 301,
+     LT = 302,
+     LE = 303,
+     EQ = 304,
+     NE = 305,
+     ALL = 306,
+     LOCALPART = 307,
+     DOMAIN = 308,
+     USER = 309,
+     DETAIL = 310,
+     RAW = 311,
+     TEXT = 312,
+     CONTENT = 313,
+     DAYS = 314,
+     ADDRESSES = 315,
+     SUBJECT = 316,
+     FROM = 317,
+     HANDLE = 318,
+     MIME = 319,
+     METHOD = 320,
+     ID = 321,
+     OPTIONS = 322,
+     LOW = 323,
+     NORMAL = 324,
+     HIGH = 325,
+     ANY = 326,
+     MESSAGE = 327,
+     INCLUDE = 328,
+     PERSONAL = 329,
+     GLOBAL = 330,
+     RETURN = 331,
+     COPY = 332
    };
 #endif
 /* Tokens.  */
@@ -131,61 +133,63 @@
 #define REMOVEFLAG 273
 #define MARK 274
 #define UNMARK 275
-#define NOTIFY 276
-#define DENOTIFY 277
-#define ANYOF 278
-#define ALLOF 279
-#define EXISTS 280
-#define SFALSE 281
-#define STRUE 282
-#define HEADER 283
-#define NOT 284
-#define SIZE 285
-#define ADDRESS 286
-#define ENVELOPE 287
-#define BODY 288
-#define COMPARATOR 289
-#define IS 290
-#define CONTAINS 291
-#define MATCHES 292
-#define REGEX 293
-#define COUNT 294
-#define VALUE 295
-#define OVER 296
-#define UNDER 297
-#define GT 298
-#define GE 299
-#define LT 300
-#define LE 301
-#define EQ 302
-#define NE 303
-#define ALL 304
-#define LOCALPART 305
-#define DOMAIN 306
-#define USER 307
-#define DETAIL 308
-#define RAW 309
-#define TEXT 310
-#define CONTENT 311
-#define DAYS 312
-#define ADDRESSES 313
-#define SUBJECT 314
-#define FROM 315
-#define HANDLE 316
-#define MIME 317
-#define METHOD 318
-#define ID 319
-#define OPTIONS 320
-#define LOW 321
-#define NORMAL 322
-#define HIGH 323
-#define ANY 324
-#define MESSAGE 325
-#define INCLUDE 326
-#define PERSONAL 327
-#define GLOBAL 328
-#define RETURN 329
-#define COPY 330
+#define HASFLAG 276
+#define FLAGS 277
+#define NOTIFY 278
+#define DENOTIFY 279
+#define ANYOF 280
+#define ALLOF 281
+#define EXISTS 282
+#define SFALSE 283
+#define STRUE 284
+#define HEADER 285
+#define NOT 286
+#define SIZE 287
+#define ADDRESS 288
+#define ENVELOPE 289
+#define BODY 290
+#define COMPARATOR 291
+#define IS 292
+#define CONTAINS 293
+#define MATCHES 294
+#define REGEX 295
+#define COUNT 296
+#define VALUE 297
+#define OVER 298
+#define UNDER 299
+#define GT 300
+#define GE 301
+#define LT 302
+#define LE 303
+#define EQ 304
+#define NE 305
+#define ALL 306
+#define LOCALPART 307
+#define DOMAIN 308
+#define USER 309
+#define DETAIL 310
+#define RAW 311
+#define TEXT 312
+#define CONTENT 313
+#define DAYS 314
+#define ADDRESSES 315
+#define SUBJECT 316
+#define FROM 317
+#define HANDLE 318
+#define MIME 319
+#define METHOD 320
+#define ID 321
+#define OPTIONS 322
+#define LOW 323
+#define NORMAL 324
+#define HIGH 325
+#define ANY 326
+#define MESSAGE 327
+#define INCLUDE 328
+#define PERSONAL 329
+#define GLOBAL 330
+#define RETURN 331
+#define COPY 332
 
 
 
@@ -195,11 +199,11 @@ typedef union YYSTYPE
 {
 
 /* Line 2068 of yacc.c  */
-#line 176 "sieve.y"
+#line 188 "sieve.y"
 
     int nval;
     char *sval;
-    stringlist_t *sl;
+    strarray_t *sl;
     test_t *test;
     testlist_t *testl;
     commandlist_t *cl;
@@ -209,11 +213,12 @@ typedef union YYSTYPE
     struct btags *btag;
     struct ntags *ntag;
     struct dtags *dtag;
+    struct ftags *ftag;
 
 
 
 /* Line 2068 of yacc.c  */
-#line 217 "y.tab.h"
+#line 222 "y.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
