@@ -40,7 +40,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: sievec.c,v 1.9 2009/03/25 23:58:04 brong Exp $
+ * $Id: sievec.c,v 1.10 2010/01/06 17:01:59 murch Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -202,81 +202,81 @@ int is_script_parsable(FILE *stream, char **errstr, sieve_script_t **ret)
     res = sieve_register_redirect(i, (sieve_callback *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_redirect() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
     res = sieve_register_discard(i, (sieve_callback *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_discard() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
     res = sieve_register_reject(i, (sieve_callback *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_reject() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
     res = sieve_register_fileinto(i, (sieve_callback *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_fileinto() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
     res = sieve_register_keep(i, (sieve_callback *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_keep() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
 
     res = sieve_register_imapflags(i, NULL);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_imapflags() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
 
     res = sieve_register_size(i, (sieve_get_size *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_size() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
   
     res = sieve_register_header(i, (sieve_get_header *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_header() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
   
     res = sieve_register_envelope(i, (sieve_get_envelope *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_envelope() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
   
     res = sieve_register_body(i, (sieve_get_body *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_body() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
   
     res = sieve_register_include(i, (sieve_get_include *) &foo);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_include() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
   
     res = sieve_register_vacation(i, &vacation);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_vacation() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
 
     res = sieve_register_notify(i, &sieve_notify);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_notify() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
 
     res = sieve_register_parse_error(i, &mysieve_error);
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_parse_error() returns %d\n", res);
-	return TIMSIEVE_FAIL;
+	goto done;
     }
 
     rewind(stream);
@@ -296,6 +296,7 @@ int is_script_parsable(FILE *stream, char **errstr, sieve_script_t **ret)
 	*errstr = NULL;
     }
 
+done:
     /* free interpreter */
     sieve_interp_free(&i);
 
