@@ -45,7 +45,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: test.c,v 1.30 2009/03/31 04:11:30 brong Exp $
+ * $Id: test.c,v 1.31 2010/01/06 17:02:00 murch Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -194,12 +194,17 @@ int parseheader(FILE *f, char **headname, char **contents) {
 		    ungetc(c, f);
 		    goto got_header;
 		}
-		/* ignore this whitespace, but we'll copy all the rest in */
-		break;
-	    } else {
-		/* just an ordinary character */
-		body[off++] = c;
+                /* http://www.faqs.org/rfcs/rfc2822.html
+		 *
+		 * > Unfolding is accomplished by simply removing any CRLF
+		 * > that is immediately followed by WSP
+		 *
+		 * So keep the actual WSP character
+		 */
 	    }
+	    /* just an ordinary character */
+	    body[off++] = c;
+	    break;
 	}
     }
 
@@ -441,12 +446,12 @@ int fileinto(void *ac, void *ic, void *sc __attribute__((unused)),
 
     printf("filing message '%s' into '%s'\n", m->name, fc->mailbox);
 
-    if (fc->imapflags->flag) {
-	int n;
-	printf("\twith flags");
-	for (n = 0; n < fc->imapflags->nflags; n++)
-	    printf(" '%s'", fc->imapflags->flag[n]);
-	printf("\n");
+    if (fc->imapflags->count) {
+	char *s = strarray_join(fc->imapflags, "' '");
+	if (s) {
+	    printf("\twith flags '%s'\n", s);
+	    free(s);
+	}
     }
 
     return (*force_fail ? SIEVE_FAIL : SIEVE_OK);
@@ -460,12 +465,12 @@ int keep(void *ac, void *ic, void *sc __attribute__((unused)),
     int *force_fail = (int*) ic;
 
     printf("keeping message '%s'\n", m->name);
-    if (kc->imapflags->flag) {
-	int n;
-	printf("\twith flags");
-	for (n = 0; n < kc->imapflags->nflags; n++)
-	    printf(" '%s'", kc->imapflags->flag[n]);
-	printf("\n");
+    if (kc->imapflags->count) {
+	char *s = strarray_join(kc->imapflags, "' '");
+	if (s) {
+	    printf("\twith flags '%s'\n", s);
+	    free(s);
+	}
     }
 
     return (*force_fail ? SIEVE_FAIL : SIEVE_OK);
@@ -557,8 +562,6 @@ sieve_vacation_t vacation = {
     &send_response		/* send_response() */
 };
 
-char *markflags[] = { "\\flagged" };
-sieve_imapflags_t mark = { markflags, 1 };
 
 struct testcase {
     int comp; 
@@ -787,6 +790,7 @@ int main(int argc, char *argv[])
     */
     int fd, res;
     struct stat sbuf;
+    static strarray_t mark = STRARRAY_INITIALIZER;
 
     while ((c = getopt(argc, argv, "v:cf")) != EOF)
 	switch (c) {
@@ -889,6 +893,7 @@ int main(int argc, char *argv[])
 	exit(1);
     }
 
+    strarray_append(&mark, "\\flagged");
     res = sieve_register_imapflags(i, &mark);
 
     if (res != SIEVE_OK) {
@@ -914,7 +919,7 @@ int main(int argc, char *argv[])
         exit(1);
     }   
 
-    res = sieve_script_load(argv[2], &exe);
+    res = sieve_script_load(script, &exe);
     if (res != SIEVE_OK) {
 	printf("sieve_script_load() returns %d\n", res);
 	exit(1);
