@@ -41,7 +41,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: actions.h,v 1.13 2009/01/14 15:50:47 murch Exp $
+ * $Id: actions.h,v 1.14 2010/01/06 17:02:01 murch Exp $
  */
 
 #ifndef _ACTIONS_H_
@@ -81,13 +81,6 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
 
 int deletescript(struct protstream *conn, mystring_t *name);
 
-/*
- * ?
- *
- */
-
-int verifyscriptname(mystring_t *name);
-
 /*
  * Is there space for this script?
  *
