@@ -41,7 +41,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: lex.c,v 1.29 2009/03/31 04:11:24 brong Exp $
+ * $Id: lex.c,v 1.30 2010/01/06 17:02:01 murch Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,18 +53,16 @@
 #include <ctype.h>
 #include <string.h>
 
-#include "prot.h"
 #include "tls.h"
 #include "lex.h"
 #include "codes.h"
-#include "mystring.h"
 #include "actions.h"
 #include "libconfig.h"
 #include "global.h"
 #include "util.h"
 #include "xmalloc.h"
 
-int token_lookup (char *str, int len __attribute__((unused)))
+static int token_lookup (char *str, int len __attribute__((unused)))
 {
     switch (*str) {
     case 'a':
@@ -82,6 +80,7 @@ int token_lookup (char *str, int len __attribute__((unused)))
     case 'g':
 	if (strcmp(str, "getscript")==0) return GETSCRIPT;
 	break;
+
     case 'h':
 	if (strcmp(str, "havespace")==0) return HAVESPACE;
 	break;
@@ -91,6 +90,10 @@ int token_lookup (char *str, int len __attribute__((unused)))
 	if (strcmp(str, "logout")==0) return LOGOUT;
 	break;
 
+    case 'n':
+	if (strcmp(str, "noop")==0) return NOOP;
+	break;
+
     case 'p':
 	if (strcmp(str, "putscript")==0) return PUTSCRIPT;
 	break;
@@ -100,6 +103,10 @@ int token_lookup (char *str, int len __attribute__((unused)))
 	if (strcmp(str, "starttls")==0 && tls_enabled())
 	    return STARTTLS;
 	break;
+
+    case 'u':
+	if (strcmp(str, "unauthenticate")==0) return UNAUTHENTICATE;
+	break;
     }
 
     /* error, nothing matched */
@@ -107,9 +114,7 @@ int token_lookup (char *str, int len __attribute__((unused)))
 }
 
 /* current state the lexer is in */
-int lexer_state = LEXER_STATE_NORMAL;
-
-extern struct protstream *sieved_out;
+static int lexer_state = LEXER_STATE_NORMAL;
 
 #define ERR() {								\
 		lexer_state=LEXER_STATE_RECOVER;                        \
@@ -121,20 +126,13 @@ extern struct protstream *sieved_out;
 		ERR();					\
   	      }
 
-int lex_reset(void)
-{
-  lexer_state = LEXER_STATE_NORMAL;
-
-  return 0;
-}
-
 void lex_setrecovering(void)
 {
   lexer_state = LEXER_STATE_RECOVER;
 }
 
-unsigned long maxscriptsize=0;
-char *buffer;
+static unsigned long maxscriptsize=0;
+static char *buffer;
 
 int lex_init(void)
 {
@@ -299,7 +297,7 @@ int timlex(mystring_t **outstr, unsigned long *outnum,  struct protstream *strea
 	unsigned long lup;
 
 	for (lup=0;lup<count;lup++)
-	  prot_getc(stream);
+	  (void)prot_getc(stream);
       }
       lexer_state=LEXER_STATE_NORMAL;
       return STRING;
@@ -363,7 +361,7 @@ int timlex(mystring_t **outstr, unsigned long *outnum,  struct protstream *strea
       if (!Uisalpha(ch)) {
 	int token;
 
-	buffer[ buff_ptr - buffer] = '\0';
+	buffer[buff_ptr - buffer] = '\0';
 
 	/* We've got the atom. */
 	token = token_lookup((char *) buffer, (int) (buff_ptr - buffer));
